#!/usr/bin/env python
"""
_CountWorkflowBySpec_

Oracle implementation of Workflow.CountWorkflowBySpec
"""

from WMCore.WMBS.MySQL.Workflow.CountWorkflowBySpec import CountWorkflowBySpec as MySQLCountWorkflowBySpec

class CountWorkflowBySpec(MySQLCountWorkflowBySpec):
    """
    Oracle version

    """



#!/usr/bin/env python
"""
_TestSpec_


Standard Spec to use for development & unit testing

"""

from WMCore.WMSpec.WMWorkload import newWorkload


def oneTaskTwoStep():
    workload = newWorkload("OneTaskTwoStepTest")
    workload.setDbsUrl("https://cmsweb-prod.cern.ch/dbs/prod/global/DBSReader")

    task1 = workload.newTask("FirstTask")

    step1 = task1.makeStep("cmsRun1")
    step1.setStepType("CMSSW")

    step2 = step1.addStep("stageOut1")
    step2.setStepType("StageOut")

    task1.applyTemplates()

    return workload


def oneTaskFourStep():
    workload = newWorkload("OneTaskFourStepTest")
    workload.setDbsUrl("https://cmsweb-prod.cern.ch/dbs/prod/global/DBSReader")

    task1 = workload.newTask("FirstTask")

    step1 = task1.makeStep("cmsRun1")
    step1.setStepType("CMSSW")

    step2 = step1.addStep("stageOut1")
    step2.setStepType("StageOut")

    step3 = step1.addStep("cmsRun2")
    step3.setStepType("CMSSW")

    step4 = step3.addStep("stageOut2")
    step4.setStepType("StageOut")

    return workload


def twoTaskTree():
    workload = newWorkload("TwoTaskTree")
    workload.setDbsUrl("https://cmsweb-prod.cern.ch/dbs/prod/global/DBSReader")

    task1 = workload.newTask("FirstTask")

    task2 = task1.addTask("SecondTask")

    step1 = task1.makeStep("cmsRun1")
    step1.setStepType("CMSSW")

    step2 = step1.addStep("stageOut1")
    step2.setStepType("StageOut")

    step3 = task2.makeStep("cmsRun2")
    step3.setStepType("CMSSW")

    step4 = step3.addStep("stageOut2")
    step4.setStepType("StageOut")

    return workload



#!/usr/bin/env python
"""
_REST_t_

Unit tests for checking RESTModel works correctly

TODO: duplicate all direct call tests to ones that use HTTP
"""

from future import standard_library
standard_library.install_aliases()

import unittest
import logging
import urllib.request, urllib.error
import json

from cherrypy import HTTPError
from nose.plugins.attrib import attr
from tempfile import NamedTemporaryFile

from WMCore_t.WebTools_t.DummyRESTModel import DummyRESTModel
from WMCore_t.WebTools_t.DummyRESTModel import (DUMMY_ROLE, DUMMY_GROUP, DUMMY_SITE)

from WMQuality.WebTools.RESTBaseUnitTest import RESTBaseUnitTest
from WMQuality.WebTools.RESTServerSetup import (DefaultConfig, cherrypySetup)
from WMQuality.WebTools.RESTClientAPI import (makeRequest, methodTest)


secureConfig = DefaultConfig('WMCore_t.WebTools_t.DummyRESTModel')
secureConfig.SecurityModule.dangerously_insecure = False
secureConfig.Webtools.environment = 'production'
tempFile = NamedTemporaryFile()
secureConfig.SecurityModule.key_file = tempFile.name
f = open(tempFile.name,"rb")
secureKey = f.read()
secureConfig.SecurityModule.section_("default")
secureConfig.SecurityModule.default.role = DUMMY_ROLE
secureConfig.SecurityModule.default.group = DUMMY_GROUP
secureConfig.SecurityModule.default.site = DUMMY_SITE

class RESTTest(RESTBaseUnitTest):
    def initialize(self):
        self.config = DefaultConfig('WMCore_t.WebTools_t.DummyRESTModel')
        self.do_debug = False
        self.do_production = False

        if self.do_production:
            self.config.Webtools.environment = 'production'
            self.config.SecurityModule.dangerously_insecure = False
            # not real keyfile but for the test.
            # file will be deleted automaticall when garbage collected.
            self.tempFile = NamedTemporaryFile()
            self.config.SecurityModule.key_file = self.tempFile.name
            self.config.SecurityModule.section_("default")
            self.config.SecurityModule.default.role = ""
            self.config.SecurityModule.default.group = ""
            self.config.SecurityModule.default.site = ""

        if not self.do_debug:
            self.config.Webtools.error_log_level = logging.WARNING
            self.config.Webtools.access_log_level = logging.WARNING

        self.urlbase = self.config.getServerUrl()

    def testGeneratorMethod(self):
        # test not accepted type should return 406 error
        url = self.urlbase + 'gen'
        output={'code':200}
        data, _ = methodTest('GET', url, accept='text/json', output=output)
        data = json.loads(data)
        self.assertEqual(type(data), list)
        self.assertEqual(type(data[0]), dict)

    def testUnsupportedFormat(self):
        # test not accepted type should return 406 error
        url = self.urlbase + 'ping'
        methodTest('GET', url, accept='text/das', output={'code':406})

    def testGoodEcho(self):
        verb ='POST'
        url = self.urlbase + 'echo'
        input_data={'message': 'unit test'}
        output={'code':200, 'type':'text/json',
              'data':'{"message": "unit test"}'}

        methodTest(verb, url, input_data, output=output)

    def testBadEchoWithPosArg(self):
        "Echo takes one argument (message), with the positional argument it should fail"
        verb ='POST'
        url = self.urlbase + 'echo/stuff'
        input_data={'message': 'unit test'}
        output={'code':400, 'type':'text/json'}
        methodTest(verb, url, input_data, output=output)

    def testBadMethodEcho(self):
        """
        The echo method isn't supported by GET, so should raise a 405
        """
        verb ='GET'
        url = self.urlbase + 'echo'
        input_data = {'data': 'unit test'}
        output = {'code':405, 'type':'text/json'}

        methodTest(verb, url, input_data, output=output)

    def testBadVerbEcho(self):
        "echo is only available to GET and POST, so should raise a 501"
        url = self.urlbase + 'echo'
        input_data = {'data': 'unit test'}
        output = {'code':501, 'type':'text/json'}

        for verb in ['DELETE']:
            methodTest(verb, url, input_data, output=output)

    def testPing(self):
        verb ='GET'
        url = self.urlbase + 'ping'
        output={'code':200, 'type':'text/json', 'data':'"ping"'}
        expireTime =3600

        methodTest(verb, url, output=output, expireTime=expireTime)

    def testBadPing(self):
        verb ='GET'

        url = self.urlbase + 'wrong'
        output={'code':404}
        methodTest(verb, url, output=output)

        url = self.urlbase + 'echo'
        output={'code':405}
        methodTest(verb, url, output=output)


        url = self.urlbase + 'ping/wrong'
        output={'code':400}
        methodTest(verb, url, output=output)

    def testException(self):
        """
        testException

        list takes a single integer argument, querying with a string
        """
        url = self.urlbase + 'list?int=a'
        self.assertRaises(urllib.error.HTTPError, urllib.request.urlopen, url)
        try:
            urllib.request.urlopen(url)
        except urllib.error.HTTPError as e:
            self.assertEqual(e.code, 400)
            self.assertEqual(e.reason, u'Bad Request')
            self.assertEqual(e.msg, u'Bad Request')
            exception_data = json.loads(e.read())
            self.assertEqual(exception_data['type'], 'HTTPError')
            self.assertEqual(exception_data['message'], 'Invalid input: Input arguments failed sanitation.')

        url = self.urlbase + 'list1?int=a'
        self.assertRaises(urllib.error.HTTPError, urllib.request.urlopen, url)
        try:
            urllib.request.urlopen(url)
        except urllib.error.HTTPError as e:
            self.assertEqual(e.code, 400)
            self.assertEqual(e.reason, u'Bad Request')
            self.assertEqual(e.msg, u'Bad Request')
            exception_data = json.loads(e.read())
            self.assertEqual(exception_data['type'], 'HTTPError')
            self.assertEqual(exception_data['message'], 'Invalid input: Arguments added where none allowed')

    def testList(self):
        verb ='GET'
        url = self.urlbase + 'list/'
        request_input = {'input_int':123, 'input_str':'abc'}
        output={'code':200, 'type':'text/json'}
        result = json.loads(methodTest(verb, url, request_input=request_input, output=output)[0])
        for i in result.keys():
            self.assertEqual(result[i], request_input[i], '%s does not match response' % i)

    def testA(self):
        # This test doesn't actually use the type, just the same thing 5 times.
        for t in ['GET', 'POST', 'PUT', 'DELETE', 'UPDATE']:
            response = makeRequest(url=self.urlbase + '/', values={'value':1234})
            assert response[1] == 200, 'Got a return code != 200 (got %s)' % response[1]

    def testSanitisePass(self):
        """
        Emulate how CherryPy passes arguments to a method, check that the data
        returned is correct.

        No server setup required
        """
        drm = DummyRESTModel(self.config.getModelConfig())

        def func(*args, **kwargs):
            sanitised_input = drm._sanitise_input(args, kwargs, "list")
            return drm.list(**sanitised_input)

        # 2 positional args (e.g. url/arg1/arg2)
        result = func(123, 'abc')
        assert result == {'input_int':123, 'input_str':'abc'},\
                                'list with 2 positional args failed: %s' % result
        # 2 query string args (e.g. url?int=arg1&str=arg2)
        result = func(input_int=123, input_str='abc')
        assert result == {'input_int':123, 'input_str':'abc'},\
                               'list with 2 query string args failed: %s' % result

        # 1 positional, 1 keyword  (e.g. url/arg1/?str=arg2)
        result = func(123, input_str='abc')
        assert result == {'input_int':123, 'input_str':'abc'},\
               'list with 1 positional, 1 keyword failed: %s' % result

    def testSanitisePassHTTP(self):
        """
        Same as testSanitisePass but do it over http and check the returned http
        codes.
        """
        # 2 positional args (e.g. url/arg1/arg2)
        url = self.urlbase + 'list/123/abc'
        response = makeRequest(url=url)
        assert response[1] == 200, \
             'list with 2 positional args failed: ' +\
             '. Got a return code != 200 (got %s)' % response[1] +\
             '. Returned data: %s' % response[0]
              # 2 query string args (e.g. url?int=arg1&str=arg2)
        url = self.urlbase + 'list/'
        response = makeRequest(url=url, values={'input_int':'123', 'input_str':'abc'})
        assert response[1] == 200, \
                 'list with 2 query string args failed: ' +\
                 '. Got a return code != 200 (got %s)' % response[1] +\
                 '. Returned data: %s' % response[0]

        # 1 positional, 1 keyword  (e.g. url/arg1/?str=arg2)
        url = self.urlbase + 'list/123/'
        response = makeRequest(url=url,
                                     values={'input_str':'abc'})
        assert response[1] == 200, \
                 'list with 1 positional, 1 keyword failed: ' +\
                 '. Got a return code != 200 (got %s)' % response[1] +\
                 '. Returned data: %s' % response[0]

    def testSanitiseAssertFail(self):
        """
        No server set up required, the purpose of the test is just
        demonstrating how validation is used.
        """
        drm = DummyRESTModel(self.config.getModelConfig())

        def func(*args, **kwargs):
            sanitised_input = drm._sanitise_input(args, kwargs, "list")
            return drm.list(**sanitised_input)

        # Wrong type for input args
        self.assertRaises(HTTPError, func, 123, 123)
        self.assertRaises(HTTPError, func, 'abc', 'abc')
        self.assertRaises(HTTPError, func, input_str = 'abc', input_int = 123, other='dfe')
        self.assertRaises(HTTPError, func, str = 123, int ='abc')
        self.assertRaises(HTTPError, func, str =' abc', int = 'abc')
        self.assertRaises(HTTPError, func, 'abc', 123)
        self.assertRaises(HTTPError, func, 'abc', 'abc')
        self.assertRaises(HTTPError, func, str = 123, int = 'abc')
        self.assertRaises(HTTPError, func, str =123, int = 123)
        self.assertRaises(HTTPError, func, str = 'abc', int ='abc')

        # Incorrect values for input args
        self.assertRaises(HTTPError, func, 1234, 'abc')
        self.assertRaises(HTTPError, func, 123, 'abcd')

        # Empty input data, when data is required
        self.assertRaises(HTTPError, func)

    def testSanitiseFailHTTP(self):
        """
        Same as testSanitisePass but do it over http and check the returned http
        codes.
        """
        # 2 positional args (e.g. url/arg1/arg2)
        url = self.urlbase + 'list/123/'
        response = makeRequest(url=url, accept='text/json')
        assert response[1] == 400, \
                 'list with 2 positional args failed: ' +\
                 '. Got a return code != 400 (got %s)' % response[1] +\
                 '. Returned data: %s' % response[0]

        self.assertEqual(response[2], 'text/json', 'type is not text/json : %s' % type)
        # 2 query string args (e.g. url?int=arg1&str=arg2)
        url = self.urlbase + 'list'
        response = makeRequest(url=url, values={'int':'abc', 'str':'abc'})
        assert response[1] == 400, \
                 'list with 2 query string args failed: ' +\
                 '. Got a return code != 400 (got %s)' % response[1] +\
                 '. Returned data: %s' % response[0]

        # 1 positional, 1 keyword  (e.g. url/arg1/?str=arg2)
        url = self.urlbase + 'list/abc'
        response = makeRequest(url=url, values={'str':'abc'})
        assert response[1] == 400, \
                 'list with 1 positional, 1 keyword failed: ' +\
                 '. Got a return code != 400 (got %s)' % response[1] +\
                 '. Returned data: %s' % response[0]

    # don't need server set up
    def testDAOBased(self):
        drm = DummyRESTModel(self.config.getModelConfig())

        result = drm.methods['GET']['data1']['call']()
        self.assertEqual( result ,  123, 'Error default value is set to 123 but returns %s' % result )

        result =  drm.methods['GET']['data2']['call'](456)
        self.assertEqual( result['num'] ,  456 )

        result =  drm.methods['GET']['data2']['call'](num = 456)
        self.assertEqual( result['num'] ,  456 )

        result =  drm.methods['GET']['data3']['call'](num = 456, thing="TEST")
        self.assertEqual( result['num'] == 456 and result['thing'] ,  "TEST" )

    # This test is flipping back and forth in Jenkins. Perhaps due to port 8888 not being available.
    # Disabling for now
    @attr("integration")
    def testDAOBasedHTTP(self):
        """
        Same as testSanitisePass but do it over http and check the returned http
        codes.
        """
        # 2 positional args (e.g. url/arg1/arg2)
        url = self.urlbase + 'data1/'
        response = makeRequest(url=url)
        assert response[1] == 200, \
                 'dao without args failed: ' +\
                 '. Got a return code != 200 (got %s)' % response[1] +\
                 '. Returned data: %s' % response[0]
        self.assertEqual( response[0] ,  '123', response[0])

        # 2 query string args (e.g. url?int=arg1&str=arg2)
        url = self.urlbase + 'data2'
        response = makeRequest(url=url, values={'num':456})

        assert response[1] == 200, \
                 'dao with 1 args failed: ' +\
                 '. Got a return code != 200 (got %s)' % response[1] +\
                 '. Returned data: %s' % response[0]
        #Warning quotation type matters
        #Should use encoded and decoded format
        self.assertEqual( response[0] ,  "{'num': '456'}", "should be {'num': '456'} but got %s" % response[0]          )

        # 1 positional, 1 keyword  (e.g. url/arg1/?str=arg2)
        url = self.urlbase + 'data3/123'
        response = makeRequest(url=url, values={'thing':'abc'})

        assert response[1] == 200, \
                 'dao with 1 positional, 1 keyword failed: ' +\
                 '. Got a return code != 200 (got %s)' % response[1] +\
                 '. Returned data: %s' % response[0]
        #Warning quotation type and order matters
        #Should use encoded and decoded format
        self.assertEqual( response[0] ,  "{'thing': 'abc', 'num': '123'}", "should be {'thing': 'abc', 'num': '123'} but got %s" % response[0] )

    def testListTypeArgs(self):
        # 2 positional args (e.g. url/arg1/arg2)
        url = self.urlbase + 'listTypeArgs?aList=1'
        response = makeRequest(url=url)
        assert response[1] == 200 and response[0] == b"[1]", \
                 'list args failed: ' +\
                 '. Got a return code != 200 (got %s)' % response[1] +\
                 '. Returned data: %s' % response[0]


        # 2 values with the same keywords (e.g. url/arg1/arg2)
        url = self.urlbase + 'listTypeArgs?aList=1&aList=2'
        response = makeRequest(url=url)
        assert response[1] == 200 and response[0] == b"[1, 2]", \
                 'list args failed: ' +\
                 '. Got a return code != 200 (got %s)' % response[1] +\
                 '. Returned data: %s' % response[0]

    @cherrypySetup(secureConfig)
    @attr("integration")
    def testAuthentication(self):
        verb ='PUT'
        url = self.urlbase + 'list1'
        urllib_data = urllib.request.urlopen(url)
        self.assertEqual(urllib_data.getcode(), 403)

        # pass proper role
        output={'code':200}
        methodTest(verb, url, output=output,
                   secure=True, secureParam={'key': secureKey,
                                             'role': DUMMY_ROLE,
                                             'group': DUMMY_GROUP,
                                             'site': DUMMY_SITE})

if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
#pylint: disable=E1101,C0103,R0902

from builtins import str, range, object

import unittest
import os
import tempfile
import shutil
from WMCore.Agent.Configuration import Configuration
from WMCore.Agent.Configuration import loadConfigurationFile
from WMCore.Agent.Configuration import saveConfigurationFile


class ConfigurationTest(unittest.TestCase):
    """
    test case for Configuration object

    """
    def setUp(self):
        """set up"""
        self.tempdir = tempfile.mkdtemp()
        self.normalSave = os.path.join(self.tempdir, "WMCore_Agent_Configuration_t_normal.py")
        self.docSave = os.path.join(self.tempdir, "WMCore_Agent_Configuration_t_documented.py")
        self.commentSave = os.path.join(self.tempdir, "WMCore_Agent_Configuration_t_commented.py")

    def tearDown(self):
        """clean up"""
        shutil.rmtree( self.tempdir )




    def testA(self):
        """ctor"""
        try:
            config = Configuration()
        except Exception as ex:
            msg = "Failed to instantiate Configuration\n"
            msg += str(ex)
            self.fail(msg)


    def testB(self):
        """add settings"""


        config = Configuration()
        config.section_("Section1")

        section1 = getattr(config, "Section1", None)
        self.assertTrue(section1 != None)

        config.section_("Section2")
        section2 = getattr(config, "Section2", None)
        self.assertTrue(section2 != None)

        self.assertRaises(AttributeError, getattr, config, "Section3")

        # basic types
        config.Section1.Parameter1 = True
        config.Section1.Parameter2 = "string"
        config.Section1.Parameter3 = 123
        config.Section1.Parameter4 = 123.456




        self.assertEqual(config.Section1.Parameter1, True)
        self.assertEqual(config.Section1.Parameter2, "string")
        self.assertEqual(config.Section1.Parameter3, 123)
        self.assertEqual(config.Section1.Parameter4, 123.456)

        # dictionary format:
        try:
            section1Dict = config.Section1.dictionary_()
        except Exception as ex:
            msg = "Error converting section to dictionary:\n"
            msg += "%s\n" % str(ex)
            self.fail(msg)

        self.assertTrue( "Parameter1" in section1Dict)
        self.assertTrue( "Parameter2" in section1Dict)
        self.assertTrue( "Parameter3" in section1Dict)
        self.assertTrue( "Parameter4" in section1Dict)

        self.assertEqual(section1Dict['Parameter1'],
                         config.Section1.Parameter1)
        self.assertEqual(section1Dict['Parameter2'],
                         config.Section1.Parameter2)
        self.assertEqual(section1Dict['Parameter3'],
                         config.Section1.Parameter3)
        self.assertEqual(section1Dict['Parameter4'],
                         config.Section1.Parameter4)


        # compound types

        config.Section2.List = ["string", 123, 123.456, False]
        config.Section2.Dictionary = { "string" : "string",
                                       "int" : 123,
                                       "float" : 123.456,
                                       "bool" : False}
        config.Section2.Tuple = ("string", 123, 123.456, False)


        self.assertEqual(config.Section2.List,
                         ["string", 123, 123.456, False])
        self.assertEqual(config.Section2.Tuple,
                         ("string", 123, 123.456, False))

        class DummyObject(object):
            pass
        # unsupported parameter type
        self.assertRaises(
            RuntimeError, setattr,
            config.Section2, "BadObject", DummyObject())
        # unsupported data type in compound type
        badList = [ DummyObject(), DummyObject()]
        self.assertRaises(
            RuntimeError, setattr,
            config.Section2, "BadList", badList)


    def testC(self):
        """add components"""

        config = Configuration()
        config.component_("Component1")
        config.component_("Component2")
        config.component_("Component3")

        comp1 = getattr(config, "Component1", None)
        self.assertTrue(comp1 != None)
        comp2 = getattr(config, "Component2", None)
        self.assertTrue(comp2 != None)


    def testD(self):
        """test documentation"""


        config = Configuration()
        config.section_("Section1")
        config.Section1.Parameter1 = True
        config.Section1.Parameter2 = "string"
        config.Section1.Parameter3 = 123
        config.Section1.Parameter4 = 123.456
        config.Section1.Parameter5 = {
            "test1" : "test2", "test3" : 123
            }


        config.Section1.document_("""This is Section1""")
        config.Section1.document_("""This is Section1.Parameter1""",
                                  "Parameter1")
        config.Section1.document_("""This is Section1.Parameter2""",
                                  "Parameter2")
        config.Section1.document_("""This is Section1.Parameter3\n with multiline comments""",
                                  "Parameter3")


        try:
            config.Section1.documentedString_()
        except Exception as ex:
            msg = "Error calling ConfigSection.documentedString_:\n"
            msg += "%s\n" % str(ex)
            self.fail(msg)
        try:
            config.Section1.commentedString_()
        except Exception as ex:
            msg = "Error calling ConfigSection.commentedString_:\n"
            msg += "%s\n" % str(ex)
            self.fail(msg)

        try:
            config.documentedString_()
        except Exception as ex:
            msg = "Error calling Configuration.documentedString_:\n"
            msg += "%s\n" % str(ex)
            self.fail(msg)
        try:
            config.commentedString_()
        except Exception as ex:
            msg = "Error calling Configuration.commentedString_:\n"
            msg += "%s\n" % str(ex)
            self.fail(msg)





    def testE(self):
        """test save/load """

        testValues = [
            "string", 123, 123.456,
            ["list", 789, 10.1 ],
            { "dict1" : "value", "dict2" : 10.0 }
            ]

        config = Configuration()
        for x in range(0, 5):
            config.section_("Section%s" % x)
            config.component_("Component%s" % x)
            sect = getattr(config, "Section%s" % x)
            comp = getattr(config, "Component%s" % x)
            sect.document_("This is Section%s" % x)
            comp.document_("This is Component%s" % x)

            for i in range(0, 5):
                setattr(comp, "Parameter%s" % i, testValues[i])
                setattr(sect, "Parameter%s" % i, testValues[i])
                comp.document_("This is Parameter%s" % i,
                               "Parameter%s" %i)
                sect.document_("This is Parameter%s" %i,
                               "Parameter%s" %i)

        stringSave = str(config)
        documentSave = config.documentedString_()
        commentSave = config.commentedString_()


        saveConfigurationFile(config, self.normalSave)
        saveConfigurationFile(config, self.docSave, document = True)
        saveConfigurationFile(config, self.commentSave, comment = True)

        plainConfig = loadConfigurationFile(self.normalSave)

        docConfig = loadConfigurationFile(self.docSave)


        commentConfig = loadConfigurationFile(self.commentSave)

        #print commentConfig.commentedString_()
        #print docConfig.documentedString_()
        #print docConfig.commentedString_()


if __name__ == '__main__':
    #config = loadConfigurationFile("/home/evansde/work/cmssrv49/WMCORE/src/python/WMCore/Agent/testConfigs/test2.py")

    #print config.Agent.wibble

    unittest.main()



#!/usr/bin/env python
"""
_ListWorkflow_

MySQL implementation of DBS3Buffer.ListWorkflow

Created on May 2, 2013

@author: dballest
"""

from WMCore.Database.DBFormatter import DBFormatter

class ListWorkflow(DBFormatter):
    """
    _ListWorkflow_

    Get a workflow id given its name and task path.
    """
    sql = """SELECT id FROM dbsbuffer_workflow
               WHERE name = :name AND task = :task"""


    def execute(self, name, task, conn = None,
                transaction = False):
        """
        _execute_

        Retrieve the workflow id
        """
        binds = {"name": name, "task" : task}
        result = self.dbi.processData(self.sql, binds, conn = conn,
                                      transaction = transaction)

        return self.formatDict(result)[0]



#!/usr/bin/env python
"""
Unittests for TemporaryEnvironment function(s)
"""

from __future__ import division

import os
import unittest

from Utils.TemporaryEnvironment import tmpEnv


class TemporaryEnvironmentTest(unittest.TestCase):
    """
    unittest for IteratorTools functions
    """

    def testSetEnv(self):
        """
        Test the grouper function (returns chunk of an iterable)
        """

        with tmpEnv(NEW_ENV_VAR=u'blah'):
            self.assertTrue('NEW_ENV_VAR' in os.environ)
            self.assertEqual(os.environ['NEW_ENV_VAR'], u'blah')
        self.assertFalse('NEW_ENV_VAR' in os.environ)


if __name__ == '__main__':
    unittest.main()



import os
import unittest
from json import loads

import nose
from Integration_t.RequestLifeCycleBase_t import RequestLifeCycleBase_t, recordException
from nose.plugins.attrib import attr

from WMCore.ACDC.DataCollectionService import DataCollectionService
from WMCore.Database.CMSCouch import Database
from WMCore.WMBase import getTestBase


class ACDCLifeCycle_t(RequestLifeCycleBase_t, unittest.TestCase):
    """Submit and track an acdc request

        Note: This test needs a previous request to be in reqmgr for it to clone
    """

    requestParams = {   'RequestType' : 'Resubmission',
                        'OriginalRequestName' : 'integration_RequestCancellation_t_121004_164419_7487',
                        'InitialTaskPath' : '/%s/DataProcessing',
                        'ACDCDatabase' : 'wmagent_acdc',
                        'Requestor' : 'integration', 'Group' : 'DMWM'
                    }

    @attr("lifecycle")
    @recordException
    def test06UploadACDC(self):
        # get previous request we can piggyback on
        for request in reversed(self.__class__.reqmgr.getRequest()):
            request = request['WMCore.ReqMgr.DataStructs.Request']['RequestName']
            if 'RequestCancellation_t' in request:
                self.__class__.requestParams['OriginalRequestName'] = request
                break
        else:
            raise nose.SkipTest("no suitable request in reqmgr to resubmit")

        self.__class__.requestParams['InitialTaskPath'] = self.__class__.requestParams['InitialTaskPath'] % self.__class__.requestParams['OriginalRequestName']
        self.__class__.requestParams['ACDCServer'] = self.__class__.endpoint + '/couchdb'

        # create and upload acdc
        service = DataCollectionService(url=self.__class__.endpoint + '/couchdb', database = 'wmagent_acdc')
        service.createCollection(self.__class__.requestParams['OriginalRequestName'], 'integration', 'DMWM')
        with open(os.path.join(getTestBase(), '..', 'data', 'ACDC', 'linacre_ACDC_ReReco13JulCosmics_120809_130020_117_120823_200309_5735.json')) as infile:
            acdc_json = infile.read().replace('linacre_ACDC_ReReco13JulCosmics_120809_130020_117_120823_200309_5735',
                                              self.__class__.requestParams['OriginalRequestName'])
        acdc_json = loads(acdc_json)
        acdc_database = Database('wmagent_acdc', self.__class__.endpoint + '/couchdb')
        acdc_database.commit(acdc_json)

# Remove tests that don't apply
delattr(RequestLifeCycleBase_t, 'test05InjectConfigs')

if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
"""
_ListJobStates_

Retrieve a list of all possible job states from WMBS.
"""




from WMCore.Database.DBFormatter import DBFormatter

class ListJobStates(DBFormatter):
    sql = "SELECT name FROM wmbs_job_state"

    def format(self, result):
        """
        _format_

        Format the results into a single list.
        """
        results = DBFormatter.format(self, result)

        resultList = []
        for result in results:
            resultList.append(result[0])

        return resultList

    def execute(self, conn = None, transaction = False):
        result = self.dbi.processData(self.sql, conn = conn,
                                      transaction = transaction)
        return self.format(result)



#!/usr/bin/env python
"""
_WMBSMergeBySize_

Generic merging for WMBS.  This will correctly handle merging files that have
been split up honoring the original file boundaries.
"""

import threading
from functools import cmp_to_key

from WMCore.DAOFactory import DAOFactory
from WMCore.DataStructs.Run import Run
from WMCore.JobSplitting.JobFactory import JobFactory
from WMCore.WMBS.File import File


def mergeUnitCompare(a, b):
    """
    _mergeUnitCompare_

    Compare two merge units.  They will be sorted first by run ID and then by
    lumi ID.
    """
    if a["run"] > b["run"]:
        return 1
    elif a["run"] == b["run"]:
        if a["lumi"] > b["lumi"]:
            return 1
        elif a["lumi"] == b["lumi"]:
            return 0
        else:
            return -1
    else:
        return -1


def fileCompare(a, b):
    """
    _fileCompare_

    Compare two files based on their "file_first_event" attribute.
    """
    if a["file_first_event"] > b["file_first_event"]:
        return 1
    if a["file_first_event"] == b["file_first_event"]:
        return 0
    else:
        return -1


def sortedFilesFromMergeUnits(mergeUnits):
    """
    _sortedFilesFromMergeUnits_

    Given a list of merge units sort them and the files that they contain.
    Return a list of sorted WMBS File structures.
    """
    mergeUnits.sort(key=cmp_to_key(mergeUnitCompare))

    sortedFiles = []
    for mergeUnit in mergeUnits:
        mergeUnit["files"].sort(key=cmp_to_key(fileCompare))

        for file in mergeUnit["files"]:
            newFile = File(id=file["file_id"], lfn=file["file_lfn"],
                           events=file["file_events"], size=file["file_size"])

            # The WMBS data structure puts locations that are passed in through
            # the constructor in the "newlocations" attribute.  We want these to
            # be in the "locations" attribute so that they get picked up by the
            # job submitter.
            newFile["locations"] = set(file["pnn"])
            newFile.addRun(Run(file["file_run"], file["file_lumi"]))
            sortedFiles.append(newFile)

    return sortedFiles


class WMBSMergeBySize(JobFactory):
    """
    _WMBSMergeBySize_

    Generic merging for WMBS.  This will correctly handle merging files that
    have been split up honoring the original file boundaries merging the files
    in the correct order.
    """

    def defineMergeUnits(self, mergeableFiles):
        """
        _defineMergeUnits_

        Split all the mergeable files into merge units.  A merge unit is a group
        of files that must be merged together.  For example, the files that
        result from event based splitting jobs need to be merged back together.
        This method will return a list of merge units.  A merge unit is a
        dictionary with the following keys: file_parent, total_events, total_size,
        run, lumi, and files.  The files in the merge group are stored in a list
        under the files key.
        """
        mergeUnits = {}
        newMergeUnit = {}

        for mergeableFile in mergeableFiles:
            newMergeFile = {}

            for key in mergeableFile:
                newMergeFile[key] = mergeableFile[key]

            if newMergeFile["pnn"] not in mergeUnits:
                mergeUnits[newMergeFile["pnn"]] = {}

            if newMergeFile["file_run"] not in mergeUnits[newMergeFile["pnn"]]:
                mergeUnits[newMergeFile["pnn"]][newMergeFile["file_run"]] = []

            for mergeUnit in mergeUnits[newMergeFile["pnn"]][newMergeFile["file_run"]]:
                if mergeUnit["file_parent"] == mergeableFile["file_parent"]:
                    mergeUnit["files"].append(newMergeFile)
                    mergeUnit["total_size"] += newMergeFile["file_size"]
                    mergeUnit["total_events"] += newMergeFile["file_events"]

                    if mergeableFile["file_run"] < mergeUnit["run"] or \
                            (mergeableFile["file_run"] == mergeUnit["run"] and \
                                         mergeableFile["file_lumi"] < mergeUnit["lumi"]):
                        newMergeUnit["run"] = newMergeFile["file_run"]
                        newMergeUnit["lumi"] = newMergeFile["file_lumi"]

                    break
            else:
                newMergeUnit = {}
                newMergeUnit["file_parent"] = newMergeFile["file_parent"]
                newMergeUnit["total_events"] = newMergeFile["file_events"]
                newMergeUnit["total_size"] = newMergeFile["file_size"]
                newMergeUnit["run"] = newMergeFile["file_run"]
                newMergeUnit["lumi"] = newMergeFile["file_lumi"]
                newMergeUnit["files"] = []
                newMergeUnit["files"].append(newMergeFile)
                mergeUnits[newMergeFile["pnn"]][newMergeFile["file_run"]].append(newMergeUnit)

        return mergeUnits

    def createMergeJob(self, mergeUnits):
        """
        _createMergeJob_

        Create a merge job for the given merge units.  All the files contained
        in the merge units will be associated to the job.
        """
        if self.currentGroup == None:
            self.newGroup()

        self.newJob(name=self.getJobName())
        sortedFiles = sortedFilesFromMergeUnits(mergeUnits)

        for file in sortedFiles:
            self.currentJob.addResourceEstimates(disk=float(file["size"]) / 1024)
            self.currentJob.addFile(file)

    def defineMergeJobs(self, mergeUnits):
        """
        _defineMergeJobs_

        Go through the list of merge units and try to combine them together into
        merge jobs that fit within the min/max filesizes and under the maximum
        number of events.
        """
        mergeJobFileSize = 0
        mergeJobEvents = 0
        mergeJobFiles = []

        for mergeUnit in mergeUnits:
            if mergeUnit["total_size"] > self.maxMergeSize or \
                            mergeUnit["total_events"] > self.maxMergeEvents:
                self.createMergeJob([mergeUnit])
                continue
            elif mergeUnit["total_size"] + mergeJobFileSize > self.maxMergeSize or \
                                    mergeUnit["total_events"] + mergeJobEvents > self.maxMergeEvents:
                if mergeJobFileSize > self.minMergeSize or \
                                self.forceMerge == True:
                    self.createMergeJob(mergeJobFiles)
                    mergeJobFileSize = 0
                    mergeJobEvents = 0
                    mergeJobFiles = []
                else:
                    continue

            mergeJobFiles.append(mergeUnit)
            mergeJobFileSize += mergeUnit["total_size"]
            mergeJobEvents += mergeUnit["total_events"]

        if mergeJobFileSize > self.minMergeSize or self.forceMerge == True:
            if len(mergeJobFiles) > 0:
                self.createMergeJob(mergeJobFiles)

        return

    def algorithm(self, *args, **kwargs):
        """
        _algorithm_

        Try to merge any available files for the subscription provided.  This
        accepts the following keyword arguments:
          max_merge_size - The maximum size of merged files
          min_merge_size - The minimum size of merged files
          max_merge_events - The maximum number of events in a merge file
        """
        # This doesn't use a proxy
        self.grabByProxy = False

        self.maxMergeSize = int(kwargs.get("max_merge_size", 1000000000))
        self.minMergeSize = int(kwargs.get("min_merge_size", 1048576))
        self.maxMergeEvents = int(kwargs.get("max_merge_events", 50000))

        myThread = threading.currentThread()
        daoFactory = DAOFactory(package="WMCore.WMBS",
                                logger=myThread.logger,
                                dbinterface=myThread.dbi)

        self.subscription["fileset"].load()

        myThread = threading.currentThread()
        if self.subscription["fileset"].open == True:
            self.forceMerge = False
        else:
            orphanDAO = daoFactory(classname="Subscriptions.FailOrphanFiles")
            orphanDAO.execute(self.subscription["id"],
                              self.subscription["fileset"].id,
                              conn=myThread.transaction.conn,
                              transaction=True)

            getAction = daoFactory(classname="Workflow.CheckInjectedWorkflow")
            injected = getAction.execute(name=self.subscription["workflow"].name,
                                         conn=myThread.transaction.conn,
                                         transaction=True)
            if injected:
                self.forceMerge = True
            else:
                self.forceMerge = False

        mergeDAO = daoFactory(classname="Subscriptions.GetFilesForMerge")
        mergeableFiles = mergeDAO.execute(self.subscription["id"],
                                          conn=myThread.transaction.conn,
                                          transaction=True)

        mergeUnits = self.defineMergeUnits(mergeableFiles)

        for pnn in mergeUnits:
            for runNumber in mergeUnits[pnn]:
                self.defineMergeJobs(mergeUnits[pnn][runNumber])

        return



#!/usr/bin/env python

"""
_TestComponentPoller_
Poller to test the skeleton
"""
from WMCore.WorkerThreads.BaseWorkerThread    import BaseWorkerThread

class TestComponentPoller(BaseWorkerThread):
    """
    _TestComponentPoller_
    """
    def __init__(self, config):
        """
        __init__
        """
        BaseWorkerThread.__init__(self)

    def algorithm(self, parameters = None):
        """
        The poller algorithm is written here
        """
        return



"""
This CherryPy thread is meant to parse all the active requests;
find which requests require parent dataset to be processed;
and run the DBS parentage lookup.
"""
from __future__ import division, print_function

from WMCore.REST.CherryPyPeriodicTask import CherryPyPeriodicTask
from WMCore.Services.DBS.DBSReader import DBS3Reader
from WMCore.Services.RequestDB.RequestDBReader import RequestDBReader
from WMCore.Services.ReqMgrAux.ReqMgrAux import ReqMgrAux


class BuildParentLock(CherryPyPeriodicTask):

    def __init__(self, rest, config):

        super(BuildParentLock, self).__init__(config)
        self.reqmgrAux = ReqMgrAux(config.reqmgr2_url, logger=self.logger)
        self.dbs = DBS3Reader(config.dbs_url)
        # cache of dbs lookups mapping input dataset to parent dataset
        self.dbsLookupCache = {}
        # set of of currently active datasets requiring parent dataset
        self.inputDatasetCache = set()
        self.reqDB = RequestDBReader(config.reqmgrdb_url)
        self.filterKeys = ['assignment-approved', 'assigned', 'staging', 'staged',
                           'failed', 'acquired', 'running-open', 'running-closed',
                           'force-complete', 'completed', 'closed-out']


    def setConcurrentTasks(self, config):
        """
        sets the list of functions which
        """
        self.concurrentTasks = [{'func': self.fetchIncludeParentsRequests,
                                 'duration': config.updateParentsInterval}]

    def fetchIncludeParentsRequests(self, config):
        """
        Fetch active requests from the "requestsincludeparents" couch view that
        have IncludeParents=True, find parents of each dataset and send to
        reqmgr2 auxiliary database.
        """
        # use this boolean to signal whether there were datasets that failed
        # to get their parentage resolved
        incompleteParentage = False
        # use this boolean to signal if new parent datasets need to be locked
        auxDbUpdateRequired = False

        setDsets = set()
        setParents = set()
        dictParents = {}

        self.logger.info("Executing parent lock cherrypy thread")

        # query couch view to find datasets for workflows requiring parent datasets
        # only returning requests with the statuses in filterKeys
        try:
            results = self.reqDB._getCouchView("requestsincludeparents", {}, self.filterKeys)
        except Exception as ex:
            self.logger.error("Error retrieving requests including parent datasets from couchdb.")
            self.logger.error("Error: %s", str(ex))
            return

        for row in results["rows"]:
            dataset = row["value"]
            setDsets.add(dataset)

        # check to see if any changes have been made
        if setDsets != self.inputDatasetCache:
            auxDbUpdateRequired = True
            self.inputDatasetCache = setDsets.copy()

        self.logger.info("Found %d unique datasets requiring the parent dataset", len(setDsets))
        if auxDbUpdateRequired:
            self.logger.info("Found new parent dataset locks to update.")
            # look up parent datasets first via the local DBS cache, if not found do lookup via DBS
            for dset in setDsets:
                if dset in self.dbsLookupCache:
                    setParents.add(self.dbsLookupCache[dset])
                    self.logger.info("Resolved parentage via lookup cache for: %s", dset)
                else:
                    try:
                        res = self.dbs.listDatasetParents(dset)
                    except Exception as exc:
                        self.logger.warning("Failed to resolve parentage for: %s. Error: %s", dset, str(exc))
                        incompleteParentage = True
                        continue
                    self.logger.info("Resolved parentage via DBS for: %s", res)
                    if res:
                        setParents.add(res[0]['parent_dataset'])
                        self.dbsLookupCache[dset] = res[0]['parent_dataset']

            if not incompleteParentage:
                dictParents['parentlocks'] = list(setParents)
                if self.reqmgrAux.updateParentLocks(dictParents):
                    self.logger.info("Parentage lookup complete and auxiliary database updated.")
                else:
                    self.logger.info("Error updating parentage document. Using stale data until next cycle.")
            else:
                # then don't replace any data for the moment, simply add new parents
                previousData = self.reqmgrAux.getParentLocks()
                # check to see if response from aux db has been populated
                if previousData and 'parentlocks' in previousData[0]:
                    setPreviousData = set(previousData[0]['parentlocks'])
                    setParents = setParents | setPreviousData
                    dictParents['parentlocks'] = list(setParents)
                    self.reqmgrAux.updateParentLocks(dictParents)
                    self.logger.info("Parentage lookup complete (with errors) and auxiliary database updated.")
                else:
                    self.logger.info("Parent locks not returned from auxiliary database. Skipping parentage update.")

        else:
            self.logger.info("No new parent datasets need locked. Skipping update of auxiliary database.")

        return



#!/usr/bin/env python
"""
_Daemon_t_

Unit tests for  daemon creation.
"""

import os
import shutil
import tempfile
import time
import unittest

from WMCore.Agent.Daemon.Create import createDaemon
from WMCore.Agent.Daemon.Details import Details
from WMQuality.TestInit import TestInit


class DaemonTest(unittest.TestCase):
    """
    _Daemon_t_

    Unit tests for message services: subscription, priority subscription, buffers,
    etc..

    """

    # minimum number of messages that need to be in queue
    _minMsg = 20
    # number of publish and gets from queue
    _publishAndGet = 10

    def setUp(self):
        "make a logger instance and create tables"
        self.testInit = TestInit(__file__)
        self.testInit.setLogging()
        self.testInit.setDatabaseConnection()
        self.tempDir = tempfile.mkdtemp()

    def tearDown(self):
        """
        Deletion of the databases
        """
        self.testInit.clearDatabase()
        shutil.rmtree(self.tempDir, True)

    def testA(self):
        """
        __testSubscribe__

        Test daemon creation
        """
        # keep the parent alive
        self.pid = createDaemon(self.tempDir, True)
        try:
            try:
                if self.pid != 0:
                    time.sleep(2)
                    details = Details(os.path.join(self.tempDir, "Daemon.xml"))
                    time.sleep(10)
                    details.killWithPrejudice()
                else:
                    while True:
                        time.sleep(1)
            except:
                pass
        finally:
            if self.pid == 0:
                os._exit(-1)
            else:
                os.system('kill -9 %s' % self.pid)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
"""
_SetFWJRPath_

MySQL implementation of Jobs.SetFWJRPath
"""




from WMCore.Database.DBFormatter import DBFormatter

class SetFWJRPath(DBFormatter):
    """
    _SetFWJRPath_

    Update the path to the framework job report for a particular job.
    """
    sql = "UPDATE wmbs_job SET fwjr_path = :fwjrpath WHERE id = :jobid"

    def execute(self, jobID = None, fwjrPath = None, conn = None,
                transaction = False, binds = None):
        """
        Send either a jobID and a path, or a list of dictionaries
        with jobid and fwjrpath values.

        """
        if not binds:
            binds = {"jobid": jobID, "fwjrpath": fwjrPath}
        self.dbi.processData(self.sql, binds,
                             conn = conn, transaction = transaction)
        return



#!/usr/bin/env python
# encoding: utf-8
"""
Interface_t.py

Created by Dave Evans on 2010-07-29.
Copyright (c) 2010 Fermilab. All rights reserved.
"""

from builtins import range
import unittest

from WMCore.GroupUser.Interface import Interface
from WMQuality.TestInitCouchApp import TestInitCouchApp
from WMCore.GroupUser.User import makeUser
from WMCore.Database.CMSCouch import Document, Database

class Interface_t(unittest.TestCase):

    def setUp(self):
        self.testInit = TestInitCouchApp(__file__)
        self.testInit.setupCouch("wmcore-groupuser-interface", "GroupUser")

        #create a test owner for the collections in this test
        self.owner1 = makeUser("DMWM", "evansde77", self.testInit.couchUrl, self.testInit.couchDbName)
        self.owner1.connect()
        self.owner1.create()

        self.owner2 = makeUser("DMWM", "drsm79", self.testInit.couchUrl, self.testInit.couchDbName)
        self.owner2.connect()
        self.owner2.create()

        #self.url = os.getenv("COUCHURL", "http://127.0.0.1:5984")
        #self.database = "awesome_acdc"

    def tearDown(self):
        self.testInit.tearDownCouch()

    def testA(self):
        """ make some documents and own them"""
        guInt = Interface(self.testInit.couchUrl, self.testInit.couchDbName)


        #create a couple of docs
        couch = Database(self.testInit.couchDbName, self.testInit.couchUrl)
        for x in range(10):
            doc = Document("document%s" % x, {"Test Data": [1,2,3,4] })
            couch.queue(doc)
        couch.commit()

        self.assertEqual(len(guInt.documentsOwned(self.owner1.group.name, self.owner1.name)), 0)
        self.assertEqual(len(guInt.documentsOwned(self.owner2.group.name, self.owner2.name)), 0)

        guInt.callUpdate("ownthis","document1", group = self.owner1.group.name, user = self.owner1.name)

        self.assertTrue("document1" in guInt.documentsOwned(self.owner1.group.name, self.owner1.name))
        self.assertEqual(len(guInt.documentsOwned(self.owner1.group.name, self.owner1.name)), 1)
        self.assertEqual(len(guInt.documentsOwned(self.owner2.group.name, self.owner2.name)), 0)

        guInt.callUpdate("ownthis","document2", group = self.owner2.group.name, user = self.owner2.name)

        self.assertTrue("document2" in guInt.documentsOwned(self.owner2.group.name, self.owner2.name))
        self.assertEqual(len(guInt.documentsOwned(self.owner1.group.name, self.owner1.name)), 1)
        self.assertEqual(len(guInt.documentsOwned(self.owner2.group.name, self.owner2.name)), 1)


        guInt.callUpdate("newgroup", "group-DataOps", group = "DataOps")

        self.assertTrue(couch.documentExists("group-DataOps") )

        guInt.callUpdate("newuser", "user-damason", group = "DataOps", user = "damason")

        self.assertTrue(couch.documentExists("user-damason") )



if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
"""
_WorkUnit.ExistsByID_

MySQL implementation of WorkUnit.ExistsByID
"""

from __future__ import absolute_import, division, print_function

from WMCore.Database.DBFormatter import DBFormatter


class ExistsByID(DBFormatter):
    """
    _WorkUnit.ExistsByID_

    MySQL implementation of WorkUnit.ExistsByID
    """

    sql = 'SELECT id FROM wmbs_workunit WHERE id = :id'

    def format(self, result):
        result = DBFormatter.format(self, result)
        if result:
            return result[0][0]
        else:
            return False

    def getBinds(self, wuid):
        return self.dbi.buildbinds(self.dbi.makelist(wuid), "id")

    def execute(self, wuid, conn=None, transaction=False):
        result = self.dbi.processData(self.sql, self.getBinds(wuid), conn=conn, transaction=transaction)
        return self.format(result)



#!/usr/bin/python

"""
_Create_

Class for creating MySQL specific schema for persistent messages.

"""

import threading

from WMCore.Database.DBCreator import DBCreator


class Create(DBCreator):
    """
    _Create_

    Class for creating MySQL specific schema for persistent messages.
    """

    def __init__(self, logger=None, dbi=None, params=None):
        myThread = threading.currentThread()
        DBCreator.__init__(self, myThread.logger, myThread.dbi)
        self.create = {}
        self.constraints = {}
        self.create['a_transaction'] = """
SET AUTOCOMMIT = 0; """
        self.create['threadpool'] = """
CREATE TABLE tp_threadpool(
   id                      int(11)      NOT NULL auto_increment,
   event                   varchar(255) NOT NULL,
   component               varchar(255) NOT NULL,
   payload                 text         NOT NULL,
   thread_pool_id          varchar(255) NOT NULL,
   state                 enum('queued','process') default 'queued',
   primary key(id)
   ) ENGINE=InnoDB ROW_FORMAT=DYNAMIC;
"""
        self.create['threadpool_buffer_in'] = """
CREATE TABLE tp_threadpool_buffer_in(
   id                      int(11)      NOT NULL auto_increment,
   event                   varchar(255) NOT NULL,
   component               varchar(255) NOT NULL,
   payload                 text         NOT NULL,
   thread_pool_id          varchar(255) NOT NULL,
   state                 enum('queued','process') default 'queued',
   primary key(id)
   ) ENGINE=InnoDB ROW_FORMAT=DYNAMIC;
"""
        self.create['threadpool_buffer_out'] = """
CREATE TABLE tp_threadpool_buffer_out(
   id                      int(11)      NOT NULL auto_increment,
   event                   varchar(255) NOT NULL,
   component               varchar(255) NOT NULL,
   payload                 text         NOT NULL,
   thread_pool_id          varchar(255) NOT NULL,
   state                 enum('queued','process') default 'queued',
   primary key(id)
   ) ENGINE=InnoDB ROW_FORMAT=DYNAMIC;
"""



#!/usr/bin/env python
"""
    _SandboxCreator_

    Given a path, workflow and task, create a sandbox within the path
"""

from builtins import map, object

from future import standard_library

from Utils.PythonVersion import PY2
from Utils.Utilities import encodeUnicodeToBytes

standard_library.install_aliases()

import logging
import os
import shutil
import tarfile
import tempfile
import zipfile

from urllib.parse import urlsplit

import PSetTweaks
import Utils
import WMCore.WMSpec.WMStep as WMStep
import WMCore.WMSpec.WMTask as WMTask
from WMCore.WMSpec.Steps.StepFactory import getFetcher


def tarFilter(tarinfo):
    """
    _tarFilter_

    Filters what goes into a tarball
    """
    if tarinfo.name.endswith(".svn") or tarinfo.name.endswith(".git"):
        return None
    else:
        return tarinfo


class SandboxCreator(object):
    def __init__(self):
        self.packageWMCore = True

    def disableWMCorePackaging(self):
        """
            __disableWMCorePackaging__

            use to keep the sandboxer from adding WMCore/* to the sandbox.
            testing would take forever otherwise
        """
        self.packageWMCore = False

    def extractSandbox(self, archivePath, targetPath):
        """
            __extractSandbox__

            extracts a sandbox at the given archivePath to the given targetPath
        """
        os.makedirs(targetPath)
        archive = tarfile.TarFile(archivePath)
        archive.extractall(targetPath)
        archive.close()

    def _makePathonPackage(self, path):
        os.makedirs(path)
        with open(path + "/__init__.py", 'w') as initHandle:
            initHandle.write("# dummy file for now")

    def makeSandbox(self, buildItHere, workload):
        """
            __makeSandbox__

            MakeSandbox creates and archives a sandbox in buildItHere,
            returning the path to the archive and putting it in the
            task
        """
        workloadName = workload.name()
        # Create path to sandbox
        pileupCachePath = "%s/pileupCache" % buildItHere
        path = "%s/%s/WMSandbox" % (buildItHere, workloadName)
        workloadFile = os.path.join(path, "WMWorkload.pkl")
        archivePath = os.path.join(buildItHere, "%s/%s-Sandbox.tar.bz2" % (workloadName, workloadName))
        # check if already built
        if os.path.exists(archivePath) and os.path.exists(workloadFile):
            workload.setSpecUrl(workloadFile)  # point to sandbox spec
            return archivePath
        if os.path.exists(path):
            shutil.rmtree(path)
        # //
        # // Set up Fetcher plugins, use default list for maintaining
        # //  compatibility
        commonFetchers = ["CMSSWFetcher", "URLFetcher", "PileupFetcher"]

        # generate the real path and make it
        self._makePathonPackage(path)

        # Add sandbox path to workload
        workload.setSandbox(archivePath)
        userSandboxes = []
        for topLevelTask in workload.taskIterator():
            for taskNode in topLevelTask.nodeIterator():
                task = WMTask.WMTaskHelper(taskNode)

                fetcherNames = commonFetchers[:]
                taskFetchers = getattr(task.data, "fetchers", [])
                fetcherNames.extend(taskFetchers)
                fetcherInstances = list(map(getFetcher, fetcherNames))

                taskPath = "%s/%s" % (path, task.name())
                self._makePathonPackage(taskPath)

                # TODO sandbox is property of workload now instead of task
                # but someother places uses as task propery (i.e. TaskQueue)
                # so backward compatability save as task attribute as well.
                setattr(task.data.input, 'sandbox', archivePath)

                for s in task.steps().nodeIterator():
                    s = WMStep.WMStepHelper(s)
                    stepPath = "%s/%s" % (taskPath, s.name())
                    self._makePathonPackage(stepPath)
                    userSandboxes.extend(s.getUserSandboxes())

                # //
                # // Execute the fetcher plugins
                # //
                for fetcher in fetcherInstances:
                    # TODO: when cache directory is set as path, cache is maintained by workflow.
                    # In that case, cache will be deleted when workflow is done,
                    # but if different workflow can share the same cache.
                    # You can set the cache direcoty somewhere else, but need to have cache refresh (delete) policy
                    fetcher.setCacheDirectory(pileupCachePath)
                    fetcher.setWorkingDirectory(taskPath)
                    fetcher(task)

        # pickle up the workload for storage in the sandbox
        workload.setSpecUrl(workloadFile)
        workload.save(workloadFile)

        # now, tar everything up and put it somewhere special

        tarContent = []
        deleteFiles = []
        tarContent.append(("%s/%s/" % (buildItHere, workloadName), '/'))

        if self.packageWMCore:

            wmcorePath = os.path.realpath(os.path.join(os.path.dirname(__file__), '..'))

            (zipHandle, zipPath) = tempfile.mkstemp()
            os.close(zipHandle)
            zipFile = zipfile.ZipFile(zipPath,
                                      mode='w',
                                      compression=zipfile.ZIP_DEFLATED)

            for (root, dirnames, filenames) in os.walk(wmcorePath):
                for filename in filenames:
                    if not filename.endswith(".svn") and not filename.endswith(".git"):
                        zipFile.write(filename=os.path.join(root, filename),
                                      # the name in the archive is the path relative to WMCore/
                                      arcname=os.path.join(root, filename)[len(wmcorePath) - len('WMCore/') + 1:])

            # Add a dummy module for zipimport testing
            (handle, dummyModulePath) = tempfile.mkstemp()
            # Python3 mkstemp file descriptors expects bytes-object
            os.write(handle, encodeUnicodeToBytes("#!/usr/bin/env python\n"))
            os.write(handle, encodeUnicodeToBytes("print('ZIPIMPORTTESTOK')\n"))
            os.close(handle)
            zipFile.write(filename=dummyModulePath, arcname='WMCore/ZipImportTestModule.py')

            # Add the wmcore zipball to the sandbox
            zipFile.close()
            tarContent.append((zipPath, '/WMCore.zip'))
            deleteFiles.append(dummyModulePath)
            deleteFiles.append(zipPath)

            psetTweaksPath = PSetTweaks.__path__[0]
            tarContent.append((psetTweaksPath, '/PSetTweaks'))

            utilsPath = Utils.__path__[0]
            tarContent.append((utilsPath, '/Utils'))

        for sb in userSandboxes:
            splitResult = urlsplit(sb)
            if not splitResult[0]:
                tarContent.append((sb, os.path.basename(sb)))

        with tarfile.open(archivePath, 'w:bz2') as tar:
            for (name, arcname) in tarContent:
                tar.add(name, arcname, filter=tarFilter)

        for deleteFile in deleteFiles:
            os.unlink(deleteFile)

        logging.info("Created sandbox %s with size %d",
                     os.path.basename(archivePath),
                     os.path.getsize(archivePath))

        return archivePath



#!/usr/bin/env python
"""
_ListOpenByName_

Oracle implementation of Fileset.ListOpenByName
"""




from WMCore.WMBS.MySQL.Fileset.ListOpenByName import ListOpenByName as ListOpenFilesetMySQL

class ListOpenByName(ListOpenFilesetMySQL):
    sql = ListOpenFilesetMySQL.sql



"""
File       : MSRuleCleanerWflow.py
Description: Provides a document Template for the MSRuleCleaner MicroServices
"""

# futures
from __future__ import division, print_function
from builtins import object, str, bytes
from future.utils import viewitems

from copy import deepcopy
from Utils.IteratorTools import flattenList


class WfParser(object):
    """
    Workflow description parser class.
    """
    def __init__(self, docSchema):
        """
        The init method for the Workflow parser class.
        :param docSchema: Document template in the form of a list of tuples as follows:
                          [('KeyName', DefaultValue, type),
                           ('KeyName', DefaultValue, type),
                           ...]
                          To be used for identifying the fields to be searched for
                          in the workflow description

        """
        self.extDoc = {}
        for tup in docSchema:
            self.extDoc[tup[0]] = {'keyName': tup[0],
                                   'values': list(),
                                   'default': tup[1],
                                   'type': tup[2]}

    def __call__(self, wfDescr):
        """
        The Call method for the Workflow parser class.
        """
        self._paramFinder(wfDescr)
        self._wfParse()
        return self.extDoc

    def _paramFinder(self, wfObj):
        """
        Private method used to recursively traverse a workflow description
        and search for all the keyNames defined in the extDoc auxiliary data
        structure. If a 'keyName' happens to be present in several nested levels,
        or in several similar objects from the same level (like {'Task1': {},
        'Task2': {} ...), all the values found are accumulated in the respective
        (flat) list at extDoc[keyName]['values'], which is later to be converted
        to the originally expected type for the given field as described in the
        Document Template
        :param wfObj: Dictionary containing the workflow description

        """
        if isinstance(wfObj, (list, set, tuple)):
            for value in wfObj:
                self._paramFinder(value)
        if isinstance(wfObj, dict):
            for key, value in viewitems(wfObj):
                self._paramFinder(value)
            for key in self.extDoc:
                if key in wfObj:
                    self.extDoc[key]['values'].append(deepcopy(wfObj[key]))

    def _wfParse(self):
        """
        Workflow description parser. Given a document template representing all the
        keyNames to be searched and a workflow description to search in recursively,
        returns all the fields that it can find aggregated according to the rules bellow:
            * if the number of found key instances is 0 - sets the default value from
              the template.
            * if the number of found key instances is 1 - sets the so found value from the
              workflow description and converts it back to the form expected and described
              in the template (removes the outermost list used for value aggregation)
            * if the number of found key instances is > 1 - the values are aggregated
              according to the expected types and data structure defined in the
              template as follows:
                * bool: sets it to True if any of the values found was set to True
                * list: chains/flattens all the sub lists into a single list containing
                        all the values found
                * dict: aggregates/flattens all the key-value pairs from all the
                        dictionaries found into one big dictionary
                        WARNING: (if an inner keyName happens to be found in multiple
                                  dictionaries from the aggregated list of dictionaries
                                  it will be overwritten with the values from the last
                                  one to be merged into the finally constructed dictionary)!
                * str:  will be accumulated in a list containing all the values found
                        WARNING: (will change the expected structure of the field from
                                  a single string to a list of strings)!

        :param wfDescr:     Dictionary with the workflow description
        :param docTemplate: Document template in the form of a list of tuples as follows:
                            [('KeyName', DefaultValue, type),
                             ('KeyName', DefaultValue, type),
                             ...]
                            To be used for identifying the fields to be searched for
                            in the workflow description
        """

        # Convert back the so aggregated extDoc to the original structure:
        for keyName, data in viewitems(self.extDoc):
            if len(data['values']) == 0:
                self.extDoc[keyName] = deepcopy(data['default'])
            elif len(data['values']) == 1:
                self.extDoc[keyName] = deepcopy(data['values'][0])
            elif len(data['values']) > 1:
                if data['type'] is bool:
                    self.extDoc[keyName] = any(data['values'])
                elif data['type'] is list:
                    self.extDoc[keyName] = list(set(flattenList(data['values'])))
                    # WARNING: If it happens this list to be constructed out of elements
                    #          which are instances of unhashable types (e.g. dict, list)
                    #          the set() call will produce an ERR, but this is unlikely
                    #          to happen, see [1] - All the fields we fetch from the
                    #          so nested structure of Task/Step Chain dictionary are
                    #          of hashable types.
                    # [1] https://github.com/dmwm/WMCore/blob/ed40d33069bdddcd98ed5b8430d5ca6662e5941f/src/python/WMCore/WMSpec/StdSpecs/StdBase.py#L1189
                elif data['type'] is dict:
                    self.extDoc[keyName] = {}
                    for item in data['values']:
                        self.extDoc[keyName].update(item)
                elif (isinstance(data['type'], tuple) and (bytes in data['type'] or str in data['type'])) or \
                     (data['type'] is bytes or data['type'] is str):
                    data['values'] = list(set(data['values']))
                    if len(data['values']) == 1:
                        self.extDoc[keyName] = deepcopy(data['values'][0])
                    else:
                        self.extDoc[keyName] = deepcopy(data['values'])


class MSRuleCleanerWflow(dict):
    """
    A minimal workflow and transfer information representation to serve the needs
    of the MSRuleCleaner Micro Service.
    """

    def __init__(self, wfDescr, **kwargs):
        super(MSRuleCleanerWflow, self).__init__(**kwargs)

        # Search for all the keys we need from the ReqManager workflow description
        wfParser = WfParser(self.docSchema())
        myDoc = wfParser(wfDescr)

        # Convert some fields to lists explicitly:
        # NOTE: Those are fields defined as strings in the original workflow
        #       representation, but may turn into lists during the recursive
        #       search and we will use them as lists for the rest of the code.
        for key in ['DataPileup', 'MCPileup', 'ParentDataset']:
            if not isinstance(myDoc[key], list):
                if myDoc[key] is None:
                    myDoc[key] = []
                else:
                    myDoc[key] = [myDoc[key]]

        self.update(myDoc)

    def docSchema(self):
        """
        Return the data schema for the document.
        It's a tuple where:
        * 1st element: is the key name / attribute in the request
        * 2nd element: is the default value
        * 3rd element: is the expected data type

        Document format:
            {
            "RequestName": "ReqName",
            "RequestType": "Type",
            "SubRequestType": "Type",
            "RequestStatus": "Status",
            "OutputDatasets": [],
            'RulesToClean': {'plineMSTrCont': [],
                             'plineMSTrBlock': [],
                             'plineAgentCont': [],
                             'plineAgentBlock': []},
            'CleanupStatus': {'plineMSTrCont': False,
                              'plineMSTrBlock': False,
                              'plineAgentCont': False,
                              'plineAgentBlock': False},
            "TransferDone": False  # information - returned by the MSOutput REST call.
            "TransferTape": False  # information - fetched by Rucio about tape rules completion
            'TargetStatus': 'normal-archived' || 'rejected-achived' || 'aborted-archived',
            'ParentageResolved': Bool,
            'PlineMarkers': None,
            'IsClean': False
            'IsLogDBClean': False,
            'IsArchivalDelayExpired': False,
            'ForceArchive': False,
            'RequestTransition': [],
            'IncludeParents': False
            'DataPileup': [],
            'MCPileup': [],
            'InputDataset': None,
            'ParentDataset': []
            }
        :return: a list of tuples
        """
        docTemplate = [
            ('RequestName', None, (bytes, str)),
            ('RequestType', None, (bytes, str)),
            ('SubRequestType', None, (bytes, str)),
            ('RequestStatus', None, (bytes, str)),
            ('OutputDatasets', [], list),
            ('RulesToClean', {}, dict),
            ('CleanupStatus', {}, dict),
            ('TransferDone', False, bool),
            ('TransferTape', False, bool),
            ('TargetStatus', None, (bytes, str)),
            ('ParentageResolved', True, bool),
            ('PlineMarkers', None, list),
            ('IsClean', False, bool),
            ('IsLogDBClean', False, bool),
            ('IsArchivalDelayExpired', False, bool),
            ('ForceArchive', False, bool),
            ('RequestTransition', [], list),
            ('IncludeParents', False, bool),
            ('DataPileup', None, (bytes, str)),
            ('MCPileup', None, (bytes, str)),
            ('InputDataset', None, (bytes, str)),
            ('ParentDataset', None, (bytes, str))]

        # NOTE: ParentageResolved is set by default to True it will be False only if:
        #       - RequestType is StepChain
        #       - The parent workflow is still in a transient status
        #       this should be one of the flags to be used to estimate if
        #       the workflow is good for archival
        return docTemplate



"""
Unit tests for MicorService/MSUnmerged/MSUnmerged.py module

"""
from __future__ import division, print_function

import json
import os
import unittest

from future.utils import viewkeys
from mock import mock

from Utils.PythonVersion import PY3
from WMCore.MicroService.MSUnmerged.MSUnmerged import MSUnmerged, MSUnmergedRSE
from WMCore.Services.Rucio import Rucio


def getTestFile(partialPath):
    """
    Returns the absolute path for the test json file
    """
    normPath = os.path.normpath(os.path.join(os.path.dirname(__file__), '..', '..', '..', '..'))
    return os.path.join(normPath, partialPath)


class RucioConMonEmul(object):
    """
    A simple class to emulate the basic behaviour of the RucioConMon Service
    """
    def __init__(self):
        super(RucioConMonEmul, self).__init__()

        self.rseUnmergedDumpFile = getTestFile('data/WMCore/MicroService/MSUnmerged/rseUnmergedDump.json')
        self.rseConsStatsDumpFile = getTestFile('data/WMCore/MicroService/MSUnmerged/rseConsStatsDump.json')
        with open(self.rseUnmergedDumpFile, encoding="utf-8") as fd:
            self.rseUnmergedDump = json.load(fd)
        with open(self.rseConsStatsDumpFile, encoding="utf-8") as fd:
            self.rseConsStatsDump = json.load(fd)

    def getRSEStats(self):
        """
        Emulates getting the latest statistics from the RucioConMon, together with the last
        update timestamps for all RSEs known to CMS Rucio.
        In reality it returns it from a file.
        """
        return self.rseConsStatsDump

    def getRSEUnmerged(self, rseName, zipped=False):
        """
        Emulates getting the list of all unmerged files in an RSE
        In reality it returns it from a file.
        """
        return self.rseUnmergedDump


def getBasicRSEData():
    """Provide a very basic rse directory structure"""
    lfns = ["/store/unmerged/alan/prod/2021/1/12/log0.tar",
            "/store/unmerged/logs/prod/2018/1/12/log1.tar",
            "/store/unmerged/logs/prod/2019/1/12/log2.tar",
            "/store/unmerged/logs/prod/2020/1/12/log3.tar",
            "/store/unmerged/logs/prod/2021/1/12/log4.tar",
            "/store/unmerged/logs/prod/2022/1/12/log5.tar",
            "/store/unmerged/data/prod/2018/1/12/log6.tar",
            "/store/unmerged/data/prod/2019/1/12/log7.tar",
            "/store/unmerged/express/prod/2020/1/12/log8.tar",
            "/store/unmerged/express/prod/2020/1/12/log9.tar",
            "/store/unmerged/alan/prod/2022/1/12/log10.tar"]
    rse = {"name": "T2_TestRSE",
           "counters": {"dirsToDeleteAll": 0},
           "dirs": {"allUnmerged": set(),
                    "toDelete": {},
                    "protected": []},
           "files": {"allUnmerged": set(lfns),
                     "toDelete": {},
                     "protected": []}
           }
    for fName in rse['files']['allUnmerged']:
        rse['dirs']['allUnmerged'].add(fName.rsplit("/", 1)[0])
    return rse


class WMStatsServerEmul(object):
    """
    A simple class to emulate the basic behaviour of the RucioConMon Service
    """
    def __init__(self):
        super(WMStatsServerEmul, self).__init__()

        self.protectedLFNsDumpFile = getTestFile('data/WMCore/MicroService/MSUnmerged/protectedLFNsDump.json')
        with open(self.protectedLFNsDumpFile, encoding="utf-8") as fd:
            self.protectedLFNsDump = json.load(fd)

    def getProtectedLFNs(self):
        return self.protectedLFNsDump


class MSUnmergedTest(unittest.TestCase):
    """ Unit test for MSUnmerged module """
    # pylint: disable=W0212,C0301

    def setUp(self):
        """ init test class """

        self.maxDiff = None

        self.msConfig = {'couch_host': 'https://cmsweb-testbed.cern.ch/couchdb',
                         'couch_wmstats_db': 'wmstats',
                         'enableRealMode': False,
                         'interval': 28800,
                         'limitRSEsPerInstance': 200,
                         'limitTiersPerInstance': ['T1', 'T2', 'T3'],
                         'limitFilesPerRSE': -1,
                         'manager': 'WMCore.MicroService.MSManager.MSManager',
                         'object': 'WMCore.MicroService.Service.RestApiHub.RestApiHub',
                         'reqmgr2Url': 'https://cmsweb-testbed.cern.ch/reqmgr2',
                         'rseExpr': 'cms_type=real&rse_type=DISK',
                         'rucioAccount': 'wmcore_transferor',
                         'rucioUrl': 'http://cms-rucio-int.cern.ch',
                         'rucioAuthUrl': 'https://cms-rucio-auth-int.cern.ch',
                         'rucioConMon': 'https://cmsweb-testbed.cern.ch/rucioconmon',
                         'services': ['unmerged'],
                         'verbose': True,
                         'wmstatsUrl': 'https://cmsweb-testbed.cern.ch/wmstatsserver',
                         'wmstatsUrlT0': "https://cmsweb-testbed.cern.ch/t0_reqmon",
                         'enableT0WMStats': False,
                         'dirFilterIncl': [],
                         'dirFilterExcl': [],
                         'emulateGfal2': True,
                         'mockMongoDB': True,
                         'mongoDB': 'msUnmergedDBUnit',
                         'mongoDBServer': 'mongodb://localhost',
                         'mongoDBUser': None,
                         'mongoDBPassword': None}

        self.creds = {"client_cert": os.getenv("X509_USER_CERT", "Unknown"),
                      "client_key": os.getenv("X509_USER_KEY", "Unknown")}
        self.rucioConfigDict = {"rucio_host": self.msConfig['rucioUrl'],
                                "auth_host": self.msConfig['rucioAuthUrl'],
                                "auth_type": "x509",
                                "account": self.msConfig['rucioAccount'],
                                "ca_cert": False,
                                "timeout": 30,
                                "request_retries": 3,
                                "creds": self.creds}

        self.msUnmerged = MSUnmerged(self.msConfig)
        self.msUnmerged.rucioConMon = RucioConMonEmul()
        self.msUnmerged.wmstatsSvc = WMStatsServerEmul()
        self.msUnmerged.resetServiceCounters()
        self.msUnmerged.rucio = Rucio.Rucio(self.msConfig['rucioAccount'],
                                            hostUrl=self.rucioConfigDict['rucio_host'],
                                            authUrl=self.rucioConfigDict['auth_host'],
                                            configDict=self.rucioConfigDict)

        super(MSUnmergedTest, self).setUp()
        if PY3:
            self.assertItemsEqual = self.assertCountEqual

    def testPlineUnmerged(self):
        # Test plineMSUnmerged:
        rse = MSUnmergedRSE('T2_US_Wisconsin')
        self.msUnmerged.rseConsStats = self.msUnmerged.rucioConMon.getRSEStats()
        self.msUnmerged.protectedLFNs = set(self.msUnmerged.wmstatsSvc.getProtectedLFNs())
        # Emulate the pipeline run while skipping the last purgeRseObj step
        self.msUnmerged.resetServiceCounters()
        rse = self.msUnmerged.updateRSETimestamps(rse, start=True, end=False)
        rse = self.msUnmerged.consRecordAge(rse)
        rse = self.msUnmerged.getUnmergedFiles(rse)
        rse = self.msUnmerged.filterUnmergedFiles(rse)
        rse = self.msUnmerged.cleanRSE(rse)
        rse = self.msUnmerged.updateServiceCounters(rse)
        rse = self.msUnmerged.updateRSETimestamps(rse, start=False, end=True)
        # self.msUnmerged.plineUnmerged.run(rse)
        expectedRSE = {'name': 'T2_US_Wisconsin',
                       'pfnPrefix': None,
                       'isClean': False,
                       'rucioConMonStatus': None,
                       'timestamps': {'endTime': mock.ANY,
                                      'prevEndTime': 0.0,
                                      'prevStartTime': 0.0,
                                      'rseConsStatTime': mock.ANY,
                                      'startTime': mock.ANY},
                       "counters": {"totalNumFiles": 11938,
                                    "totalNumDirs": 11,
                                    "dirsToDelete": 6,
                                    "filesToDelete": 0,
                                    "filesDeletedSuccess": 0,
                                    "filesDeletedFail": 0,
                                    "dirsDeletedSuccess": 0,
                                    "dirsDeletedFail": 0,
                                    "gfalErrors": {}},
                       'files': {'allUnmerged': mock.ANY,
                                 'deletedFail': set(),
                                 'deletedSuccess': set(),
                                 'protected': {},
                                 'toDelete': {'/store/unmerged/Phase2HLTTDRSummer20ReRECOMiniAOD/DYToLL_M-50_TuneCP5_14TeV-pythia8/FEVT/FlatPU0To200_pilot_111X_mcRun4_realistic_T15_v1-v2': mock.ANY,
                                              '/store/unmerged/Run2016G/DoubleEG/MINIAOD/UL2016_MiniAODv2-v1': mock.ANY,
                                              '/store/unmerged/SAM/testSRM/SAM-cms-lvs-gridftp.hep.wisc.edu': mock.ANY,
                                              '/store/unmerged/SAM/testSRM/SAM-cms-lvs-gridftp.hep.wisc.edu/lcg-util': mock.ANY,
                                              '/store/unmerged/SAM/testSRM/SAM-cmssrm.hep.wisc.edu': mock.ANY,
                                              '/store/unmerged/SAM/testSRM/SAM-cmssrm.hep.wisc.edu/lcg-util': mock.ANY}},
                       'dirs': {'allUnmerged': set(),
                                "deletedSuccess": set(),
                                "deletedFail": set(),
                                'protected': {'/store/unmerged/RunIIAutumn18FSPremix/PMSSM_set_1_prompt_1_TuneCP2_13TeV-pythia8/AODSIM/GridpackScan_102X_upgrade2018_realistic_v15-v1',
                                              '/store/unmerged/RunIIFall17DRPremix/Suu_Diquark_S4000_chi1160_TuneCP2_13TeV-madgraph-pythia8/AODSIM/PU2017_94X_mc2017_realistic_v11-v1',
                                              '/store/unmerged/RunIIFall17DRPremix/Suu_Diquark_S4000_chi680_TuneCP2_13TeV-madgraph-pythia8/AODSIM/PU2017_94X_mc2017_realistic_v11-v1',
                                              '/store/unmerged/RunIISummer20UL16HLTAPV/QCD_Pt-20To30_MuEnrichedPt5_TuneCP5_13TeV_pythia8/GEN-SIM-RAW/80X_mcRun2_asymptotic_2016_TrancheIV_v6-v1',
                                              '/store/unmerged/RunIISummer20UL16SIMAPV/ZJetsToQQ_HT-400to600_TuneCP5_13TeV-madgraphMLM-pythia8/GEN-SIM/106X_mcRun2_asymptotic_preVFP_v8-v1'},
                                'toDelete': {'/store/unmerged/Phase2HLTTDRSummer20ReRECOMiniAOD/DYToLL_M-50_TuneCP5_14TeV-pythia8/FEVT/FlatPU0To200_pilot_111X_mcRun4_realistic_T15_v1-v2',
                                             '/store/unmerged/Run2016G/DoubleEG/MINIAOD/UL2016_MiniAODv2-v1',
                                             '/store/unmerged/SAM/testSRM/SAM-cms-lvs-gridftp.hep.wisc.edu',
                                             '/store/unmerged/SAM/testSRM/SAM-cms-lvs-gridftp.hep.wisc.edu/lcg-util',
                                             '/store/unmerged/SAM/testSRM/SAM-cmssrm.hep.wisc.edu',
                                             '/store/unmerged/SAM/testSRM/SAM-cmssrm.hep.wisc.edu/lcg-util'}}
                       }
        self.assertDictEqual(rse, expectedRSE)

    def testCutPath(self):
        filePath = '/store/unmerged/SAM/testSRM/SAM-cmssrm.hep.wisc.edu/lcg-util/testfile-put-nospacetoken-1502337521-08cc70247c3f.txt'
        expectedFilePath = '/store/unmerged/SAM/testSRM/SAM-cmssrm.hep.wisc.edu/lcg-util'
        self.assertEqual(self.msUnmerged._cutPath(filePath), expectedFilePath)

        filePath = '/store/unmerged/SAM/testSRM/'
        expectedFilePath = '/store/unmerged/SAM/testSRM'
        self.assertEqual(self.msUnmerged._cutPath(filePath), expectedFilePath)

        filePath = '/store/unmerged/RunIIAutumn18FSPremix/PMSSM_set_1_prompt_1_TuneCP2_13TeV-pythia8/AODSIM/GridpackScan_102X_upgrade2018_realistic_v15-v1/00001/2A335139-C39F-F94C-9D06-D0D32296C62E.root'
        expectedFilePath = '/store/unmerged/RunIIAutumn18FSPremix/PMSSM_set_1_prompt_1_TuneCP2_13TeV-pythia8/AODSIM/GridpackScan_102X_upgrade2018_realistic_v15-v1'
        self.assertEqual(self.msUnmerged._cutPath(filePath), expectedFilePath)

    def testFilterInclDirectories(self):
        "Test MSUnmerged with including directories filter"
        toDeleteDict = {"/store/unmerged/data/prod/2018/1/12": ["/store/unmerged/data/prod/2018/1/12/log6.tar"],
                        "/store/unmerged/express/prod/2020/1/12": ["/store/unmerged/express/prod/2020/1/12/log8.tar",
                                                                   "/store/unmerged/express/prod/2020/1/12/log9.tar"]}
        rseData = getBasicRSEData()

        self.msUnmerged.msConfig['dirFilterIncl'] = ["/store/unmerged/data/prod/2018/",
                                                     "/store/unmerged/express"]
        self.msUnmerged.protectedLFNs = set()
        filterData = self.msUnmerged.filterUnmergedFiles(rseData)
        self.assertEqual(filterData['counters']['dirsToDelete'], 2)
        self.assertItemsEqual(viewkeys(filterData['files']['toDelete']), viewkeys(toDeleteDict))
        self.assertItemsEqual(list(filterData['files']['toDelete']['/store/unmerged/data/prod/2018/1/12']),
                              toDeleteDict['/store/unmerged/data/prod/2018/1/12'])
        self.assertItemsEqual(list(filterData['files']['toDelete']['/store/unmerged/express/prod/2020/1/12']),
                              toDeleteDict['/store/unmerged/express/prod/2020/1/12'])

    def testFilterExclDirectories(self):
        "Test MSUnmerged with excluding directories filter"
        toDeleteDict = {"/store/unmerged/data/prod/2018/1/12": ["/store/unmerged/data/prod/2018/1/12/log6.tar"],
                        "/store/unmerged/express/prod/2020/1/12": ["/store/unmerged/express/prod/2020/1/12/log8.tar",
                                                                   "/store/unmerged/express/prod/2020/1/12/log9.tar"]}
        rseData = getBasicRSEData()

        self.msUnmerged.msConfig['dirFilterExcl'] = ["/store/unmerged/logs",
                                                     "/store/unmerged/data/prod/2019",
                                                     "/store/unmerged/alan/prod"]
        self.msUnmerged.protectedLFNs = set()
        filterData = self.msUnmerged.filterUnmergedFiles(rseData)
        self.assertEqual(filterData['counters']['dirsToDelete'], 2)
        self.assertItemsEqual(viewkeys(filterData['files']['toDelete']), viewkeys(toDeleteDict))
        self.assertItemsEqual(list(filterData['files']['toDelete']['/store/unmerged/data/prod/2018/1/12']),
                              toDeleteDict['/store/unmerged/data/prod/2018/1/12'])
        self.assertItemsEqual(list(filterData['files']['toDelete']['/store/unmerged/express/prod/2020/1/12']),
                              toDeleteDict['/store/unmerged/express/prod/2020/1/12'])

    def testFilterInclExclDirectories(self):
        "Test MSUnmerged with including and excluding directories filter"
        toDeleteDict = {"/store/unmerged/express/prod/2020/1/12": ["/store/unmerged/express/prod/2020/1/12/log8.tar",
                                                                   "/store/unmerged/express/prod/2020/1/12/log9.tar"]}
        rseData = getBasicRSEData()
        self.msUnmerged.msConfig['dirFilterIncl'] = ["/store/unmerged/data/prod/2018/",
                                                     "/store/unmerged/express"]
        self.msUnmerged.msConfig['dirFilterExcl'] = ["/store/unmerged/logs",
                                                     "/store/unmerged/data/prod",
                                                     "/store/unmerged/alan/prod"]
        self.msUnmerged.protectedLFNs = set()
        filterData = self.msUnmerged.filterUnmergedFiles(rseData)
        self.assertEqual(filterData['counters']['dirsToDelete'], 1)
        self.assertItemsEqual(viewkeys(filterData['files']['toDelete']), viewkeys(toDeleteDict))
        self.assertItemsEqual(list(filterData['files']['toDelete']['/store/unmerged/express/prod/2020/1/12']),
                              toDeleteDict['/store/unmerged/express/prod/2020/1/12'])



#!/usr/bin/env python
"""
The actual taskArchiver algorithm
"""
from __future__ import print_function
__all__ = []



import threading
import logging
import time

from WMCore.WorkerThreads.BaseWorkerThread import BaseWorkerThread
from WMCore.WorkQueue.WorkQueue import WorkQueue, localQueue

class WMAgentTasks(BaseWorkerThread):
    """
    Polls for Error Conditions, handles them
    """
    def __init__(self, resources):
        """
        Initialise class members
        """
        BaseWorkerThread.__init__(self)
        self.resources = resources
        self.wq = localQueue()

    def algorithm(self, parameters):
        """
        """

        data = self.wq.getWork(self.resources, {})
        print("Data back from workqueue")
        print("%s: %s" % (self.resources, data))
        if len(data) != 0:
            elementIDs = []
            for element in data:
                elementIDs.append(element['element_id'])
            time.sleep(5)
            self.wq.doneWork(elementIDs)
            #self.wq.failWork(elementIDs)
            #self.wq.cancelWork(elementIDs)



#!/usr/bin/env python
"""
_FailOrphanFiles_

Oracle implementation of Subscriptions.FailOrphanFiles
"""

from WMCore.WMBS.MySQL.Subscriptions.FailOrphanFiles import FailOrphanFiles as MySQLFailOrphanFiles

class FailOrphanFiles(MySQLFailOrphanFiles):
    pass



"""
Created on May 19, 2015
"""

from __future__ import (division, print_function)
import json
from WMCore.REST.CherryPyPeriodicTask import CherryPyPeriodicTask
from WMCore.Services.ReqMgrAux.ReqMgrAux import ReqMgrAux
from WMCore.Services.pycurl_manager import RequestHandler


class AuxCacheUpdateTasks(CherryPyPeriodicTask):
    """
    Updates Aux db update periodically. (i.e. TagCollector)
    """

    def __init__(self, rest, config):

        super(AuxCacheUpdateTasks, self).__init__(config)
        self.reqmgrAux = ReqMgrAux(config.reqmgr2_url, logger=self.logger)
        self.mgr = RequestHandler()

    def setConcurrentTasks(self, config):
        """
        sets the list of functions which
        """
        self.concurrentTasks = [{'func': self.updateAuxiliarDocs, 'duration': config.tagCollectDuration}]

    def updateAuxiliarDocs(self, config):
        """
        Update the central couch database with auxiliary documents
        that need to be constanly updated whenever an update is
        made at the data source
        """
        self.logger.info("Updating auxiliary couch documents ...")

        self.reqmgrAux.populateCMSSWVersion(config.tagcollect_url, **config.tagcollect_args)

        try:
            data = self.mgr.getdata(config.unified_url, params={},
                                    headers={'Accept': 'application/json'})
            data = json.loads(data)
        except Exception as ex:
            msg = "Failed to retrieve unified configuration from github. Error: %s" % str(ex)
            msg += "\nRetrying again in the next cycle"
            self.logger.error(msg)
            return

        self.reqmgrAux.updateUnifiedConfig(data, docName="config")



#!/usr/bin/env python
"""
_SetLocation_

MySQL implementation of DBSBuffer.SetLocation
"""




from WMCore.Database.DBFormatter import DBFormatter

class SetLocation(DBFormatter):
    sql = """INSERT IGNORE INTO dbsbuffer_file_location (filename, location)
               VALUES (:fileid, :locationid)"""

    def execute(self, binds, conn = None, transaction = None):
        self.dbi.processData(self.sql, binds, conn = conn,
                             transaction = transaction)
        return



#!/usr/bin/env python
"""
_SizeBased_t_

Size based splitting test.
"""

import unittest
import threading
import os

from WMCore.WMBS.File import File
from WMCore.WMBS.Fileset import Fileset
from WMCore.WMBS.Job import Job
from WMCore.WMBS.Subscription import Subscription
from WMCore.WMBS.Workflow import Workflow

from WMCore.JobSplitting.SplitterFactory import SplitterFactory
from WMCore.Services.UUIDLib import makeUUID
from WMCore.DAOFactory import DAOFactory
from WMCore.WMFactory import WMFactory
from WMQuality.TestInit import TestInit

class SizeBasedTest(unittest.TestCase):
    """
    _SizeBasedTest_

    Test size based job splitting.
    """

    def setUp(self):
        """
        _setUp_

        Create two subscriptions: One that contains a single file and one that
        contains multiple files.
        """
        self.testInit = TestInit(__file__)
        self.testInit.setLogging()
        self.testInit.setDatabaseConnection()
        self.testInit.setSchema(customModules = ["WMCore.WMBS"],
                                useDefault = False)

        myThread = threading.currentThread()
        daofactory = DAOFactory(package = "WMCore.WMBS",
                                logger = myThread.logger,
                                dbinterface = myThread.dbi)

        locationAction = daofactory(classname = "Locations.New")
        locationAction.execute(siteName = "site1", pnn = "T2_CH_CERN")
        locationAction.execute(siteName = "site2", pnn = "T1_US_FNAL_Disk")

        self.multipleFileFileset = Fileset(name = "TestFileset1")
        self.multipleFileFileset.create()
        for i in range(10):
            newFile = File(makeUUID(), size = 1000, events = 100,
                           locations = set(["T2_CH_CERN"]))
            newFile.create()
            self.multipleFileFileset.addFile(newFile)
        self.multipleFileFileset.commit()

        self.singleFileFileset = Fileset(name = "TestFileset2")
        self.singleFileFileset.create()
        newFile = File("/some/file/name", size = 1000, events = 100,
                       locations = set(["T2_CH_CERN"]))
        newFile.create()
        self.singleFileFileset.addFile(newFile)
        self.singleFileFileset.commit()


        self.multipleSiteFileset = Fileset(name = "TestFileset3")
        self.multipleSiteFileset.create()
        for i in range(5):
            newFile = File(makeUUID(), size = 1000, events = 100)
            newFile.setLocation("T2_CH_CERN")
            newFile.create()
            self.multipleSiteFileset.addFile(newFile)
        for i in range(5):
            newFile = File(makeUUID(), size = 1000, events = 100)
            newFile.setLocation(["T2_CH_CERN","T1_US_FNAL_Disk"])
            newFile.create()
            self.multipleSiteFileset.addFile(newFile)
        self.multipleSiteFileset.commit()

        testWorkflow = Workflow(spec = "spec.xml", owner = "Steve",
                                name = "wf001", task="Test")
        testWorkflow.create()
        self.multipleFileSubscription = Subscription(fileset = self.multipleFileFileset,
                                                     workflow = testWorkflow,
                                                     split_algo = "SizeBased",
                                                     type = "Processing")
        self.multipleFileSubscription.create()
        self.singleFileSubscription = Subscription(fileset = self.singleFileFileset,
                                                   workflow = testWorkflow,
                                                   split_algo = "SizeBased",
                                                   type = "Processing")
        self.singleFileSubscription.create()
        self.multipleSiteSubscription = Subscription(fileset = self.multipleSiteFileset,
                                                     workflow = testWorkflow,
                                                     split_algo = "SizeBased",
                                                     type = "Processing")
        self.multipleSiteSubscription.create()
        return

    def tearDown(self):
        """
        _tearDown_

        Clear out WMBS.
        """
        self.testInit.clearDatabase()
        return

    def testExactEvents(self):
        """
        _testExactEvents_

        Test event based job splitting when the number of events per job is
        exactly the same as the number of events in the input file.
        """

        splitter = SplitterFactory()
        jobFactory = splitter(self.singleFileSubscription)

        jobGroups = jobFactory(size_per_job = 1000)

        assert len(jobGroups) == 1, \
               "ERROR: JobFactory didn't return one JobGroup."

        assert len(jobGroups[0].jobs) == 1, \
               "ERROR: JobFactory didn't create a single job."

        job = jobGroups[0].jobs.pop()

        assert job.getFiles(type = "lfn") == ["/some/file/name"], \
               "ERROR: Job contains unknown files."


        return


    def testMultipleFiles(self):
        """
        _testMultipleFiles_

        Tests the mechanism for splitting up multiple files into jobs with
        a variety of different arguments.
        """

        splitter   = SplitterFactory()
        jobFactory = splitter(self.multipleFileSubscription)

        jobGroups  = jobFactory(size_per_job = 1000)

        self.assertEqual(len(jobGroups), 1)
        self.assertEqual(len(jobGroups[0].jobs), 10)
        for job in jobGroups[0].jobs:
            self.assertEqual(len(job.getFiles()), 1)

        return


    def testMultipleFiles2000(self):
        """
        _testMultipleFiles2000_

        Tests the mechanism for splitting up multiple files into jobs with
        a variety of different arguments.
        """

        splitter   = SplitterFactory()
        jobFactory = splitter(self.multipleFileSubscription)
        #Test it with two files per job
        jobGroups  = jobFactory(size_per_job = 2000)

        self.assertEqual(len(jobGroups), 1)
        self.assertEqual(len(jobGroups[0].jobs), 5)
        for job in jobGroups[0].jobs:
            self.assertEqual(len(job.getFiles()), 2)

        return


    def testMultipleFiles2500(self):
        """
        _testMultipleFiles2500_

        Tests the mechanism for splitting up multiple files into jobs with
        a variety of different arguments.
        """

        splitter   = SplitterFactory()
        jobFactory = splitter(self.multipleFileSubscription)


        #Now test it with a size that can't be broken up evenly
        jobGroups  = jobFactory(size_per_job = 2500)

        self.assertEqual(len(jobGroups), 1)
        self.assertEqual(len(jobGroups[0].jobs), 5)
        for job in jobGroups[0].jobs:
            self.assertEqual(len(job.getFiles()), 2)

        return


    def testMultipleFiles500(self):
        """
        _testMultipleFiles500_

        Tests the mechanism for splitting up multiple files into jobs with
        a variety of different arguments.
        """

        splitter   = SplitterFactory()
        jobFactory = splitter(self.multipleFileSubscription)


        #Test it with something too small to handle; should return one job per file, plus one extra
        #open at the end
        jobGroups  = jobFactory(size_per_job = 500)

        self.assertEqual(len(jobGroups), 1)
        self.assertEqual(len(jobGroups[0].jobs), 10)

        return


    def testMultipleSites(self):
        """
        _testMultipleSites_

        Tests how to break up files at different locations
        """

        splitter   = SplitterFactory()
        jobFactory = splitter(self.multipleSiteSubscription)

        jobGroups  = jobFactory(size_per_job = 1000)

        self.assertEqual(len(jobGroups), 2)
        self.assertEqual(len(jobGroups[0].jobs), 5)
        for job in jobGroups[0].jobs:
            self.assertEqual(len(job.getFiles()), 1)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
"""
_SizeBased_t_

Size based splitting test.
"""




from builtins import range
import unittest

from WMCore.DataStructs.File import File
from WMCore.DataStructs.Fileset import Fileset
from WMCore.DataStructs.Job import Job
from WMCore.DataStructs.Subscription import Subscription
from WMCore.DataStructs.Workflow import Workflow

from WMCore.JobSplitting.SplitterFactory import SplitterFactory
from WMCore.Services.UUIDLib import makeUUID

class SizeBasedTest(unittest.TestCase):
    def setUp(self):
        """
        _setUp_

        Create two subscriptions: One that contains a single file and one that
        contains multiple files.
        """
        self.multipleFileFileset = Fileset(name = "TestFileset1")
        for i in range(10):
            newFile = File(makeUUID(), size = 1000, events = 100, locations = set(["somese.cern.ch"]))
            self.multipleFileFileset.addFile(newFile)

        self.singleFileFileset = Fileset(name = "TestFileset2")
        newFile = File("/some/file/name", size = 1000, events = 100, locations = set(["somese.cern.ch"]))
        self.singleFileFileset.addFile(newFile)

        self.multipleSiteFileset = Fileset(name = "TestFileset3")
        for i in range(5):
            newFile = File(makeUUID(), size = 1000, events = 100, locations = set(["somese.cern.ch"]))
            newFile.setLocation("somese.cern.ch")
            self.multipleSiteFileset.addFile(newFile)
        for i in range(5):
            newFile = File(makeUUID(), size = 1000, events = 100)
            newFile.setLocation(["somese.cern.ch","otherse.cern.ch"])
            self.multipleSiteFileset.addFile(newFile)

        testWorkflow = Workflow()
        self.multipleFileSubscription = Subscription(fileset = self.multipleFileFileset,
                                                     workflow = testWorkflow,
                                                     split_algo = "SizeBased",
                                                     type = "Processing")
        self.singleFileSubscription = Subscription(fileset = self.singleFileFileset,
                                                   workflow = testWorkflow,
                                                   split_algo = "SizeBased",
                                                   type = "Processing")
        self.multipleSiteSubscription = Subscription(fileset = self.multipleSiteFileset,
                                                     workflow = testWorkflow,
                                                     split_algo = "EventBased",
                                                     type = "Processing")
        return

    def tearDown(self):
        """
        _tearDown_

        Nothing to do...
        """
        pass

    def testExactEvents(self):
        """
        _testExactEvents_

        Test event based job splitting when the number of events per job is
        exactly the same as the number of events in the input file.
        """

        splitter = SplitterFactory()
        jobFactory = splitter(self.singleFileSubscription)

        jobGroups = jobFactory(size_per_job = 1000)

        assert len(jobGroups) == 1, \
               "ERROR: JobFactory didn't return one JobGroup."

        assert len(jobGroups[0].jobs) == 1, \
               "ERROR: JobFactory didn't create a single job."

        job = jobGroups[0].jobs.pop()

        assert job.getFiles(type = "lfn") == ["/some/file/name"], \
               "ERROR: Job contains unknown files."


        return


    def testFiles1000(self):
        """
        _testMultipleFiles_

        Tests the mechanism for splitting up multiple files into jobs with
        a variety of different arguments.
        """

        splitter   = SplitterFactory()
        jobFactory = splitter(self.multipleFileSubscription)

        jobGroups  = jobFactory(size_per_job = 1000)

        self.assertEqual(len(jobGroups), 1)
        self.assertEqual(len(jobGroups[0].jobs), 10)
        for job in jobGroups[0].jobs:
            self.assertEqual(len(job.getFiles()), 1)

        return


    def testFiles2000(self):
        """
        _testMultipleFiles_

        Tests the mechanism for splitting up multiple files into jobs with
        a variety of different arguments.
        """
        splitter   = SplitterFactory()
        jobFactory = splitter(self.multipleFileSubscription)

        #Test it with two files per job
        jobGroups  = jobFactory(size_per_job = 2000)

        self.assertEqual(len(jobGroups), 1)
        self.assertEqual(len(jobGroups[0].jobs), 5)
        for job in jobGroups[0].jobs:
            self.assertEqual(len(job.getFiles()), 2)

        return


    def testFiles2500(self):
        """
        _testMultipleFiles_

        Tests the mechanism for splitting up multiple files into jobs with
        a variety of different arguments.
        """
        splitter   = SplitterFactory()
        jobFactory = splitter(self.multipleFileSubscription)


        #Now test it with a size that can't be broken up evenly
        jobGroups  = jobFactory(size_per_job = 2500)

        self.assertEqual(len(jobGroups), 1)
        self.assertEqual(len(jobGroups[0].jobs), 5)
        for job in jobGroups[0].jobs:
            self.assertEqual(len(job.getFiles()), 2)

        return


    def testFiles500(self):
        """
        _testMultipleFiles_

        Tests the mechanism for splitting up multiple files into jobs with
        a variety of different arguments.
        """
        splitter   = SplitterFactory()
        jobFactory = splitter(self.multipleFileSubscription)


        #Test it with something too small to handle; should return one job per file
        jobGroups  = jobFactory(size_per_job = 500)
        self.assertEqual(len(jobGroups), 1)
        self.assertEqual(len(jobGroups[0].jobs), 10)

        return



if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

"""
_scaleTest_

Fills a DBSBuffer with random data and then runs a DBSUpload process on it.
"""
from __future__ import print_function
import os
import sys
import time
import random
import logging
import threading

from WMCore.Services.UUIDLib       import makeUUID
from WMQuality.TestInit         import TestInit
from WMQuality.Emulators        import EmulatorSetup
from WMCore.Agent.Configuration import Configuration
from WMCore.DAOFactory          import DAOFactory
from WMCore.DataStructs.Run     import Run

from WMComponent.DBS3Buffer.DBSBufferFile   import DBSBufferFile
from WMComponent.DBS3Buffer.DBSBufferUtil   import DBSBufferUtil
from WMComponent.DBS3Buffer.DBSUploadPoller import DBSUploadPoller

class scaleTestFiller:
    """
    _scaleTestFiller_

    Initializes the DB and the DBSUploader
    On __call__() it creates data and uploads it.
    """

    def __init__(self):
        """
        __init__

        Init the DB
        """

        self.testInit = TestInit(__file__)
        self.testInit.setLogging()
        self.testInit.setDatabaseConnection(destroyAllDatabase = True)
        self.testInit.setSchema(customModules = ["WMComponent.DBS3Buffer"],
                                useDefault = False)
        self.configFile = EmulatorSetup.setupWMAgentConfig()

        myThread = threading.currentThread()
        self.bufferFactory = DAOFactory(package = "WMComponent.DBS3Buffer",
                                        logger = myThread.logger,
                                        dbinterface = myThread.dbi)

        locationAction = self.bufferFactory(classname = "DBSBufferFiles.AddLocation")
        locationAction.execute(siteName = "se1.cern.ch")
        locationAction.execute(siteName = "se1.fnal.gov")
        locationAction.execute(siteName = "malpaquet")

        config = self.getConfig()
        self.dbsUploader = DBSUploadPoller(config = config)

        return


    def __call__(self):
        """
        __call__

        Generate some random data
        """

        # Generate somewhere between one and a thousand files
        name = "ThisIsATest_%s" % (makeUUID())
        nFiles = random.randint(10, 2000)
        name = name.replace('-', '_')
        name = '%s-v0' % name
        files = self.getFiles(name = name, nFiles = nFiles)

        print("Inserting %i files for dataset %s" % (nFiles * 2, name))

        try:
            self.dbsUploader.algorithm()
        except:
            self.dbsUploader.close()
            raise

        # Repeat just to make sure
        try:
            self.dbsUploader.algorithm()
        except:
            self.dbsUploader.close()
            raise


        return



    def getConfig(self):
        """
        _getConfig_

        This creates the actual config file used by the component

        """


        config = self.testInit.getConfiguration()
        self.testInit.generateWorkDir(config)

        #First the general stuff
        config.section_("General")
        config.General.workDir = os.getenv("TESTDIR", os.getcwd())

        config.section_("Agent")
        config.Agent.componentName = 'DBSUpload'
        config.Agent.useHeartbeat  = False

        #Now the CoreDatabase information
        #This should be the dialect, dburl, etc
        config.section_("CoreDatabase")
        config.CoreDatabase.connectUrl = os.getenv("DATABASE")
        config.CoreDatabase.socket     = os.getenv("DBSOCK")


        config.component_("DBS3Upload")
        config.DBS3Upload.pollInterval     = 10
        config.DBS3Upload.logLevel         = 'DEBUG'
        config.DBS3Upload.DBSBlockMaxFiles = 500
        config.DBS3Upload.DBSBlockMaxTime  = 600
        config.DBS3Upload.DBSBlockMaxSize  = 999999999999
        config.DBS3Upload.dbsUrl           = 'http://cms-xen40.fnal.gov:8787/dbs/prod/global/DBSWriter'
        config.DBS3Upload.namespace        = 'WMComponent.DBS3Buffer.DBS3Upload'
        config.DBS3Upload.componentDir     = os.path.join(os.getcwd(), 'Components')
        config.DBS3Upload.nProcesses       = 1
        config.DBS3Upload.dbsWaitTime      = 1

        return config



    def getFiles(self, name, tier = 'RECO', nFiles = 12, site = "malpaquet", nLumis = 1):
        """
        Create some quick dummy test files

        """

        files = []

        for f in range(nFiles):
            testFile = DBSBufferFile(lfn = '/data/store/random/random/RANDOM/test/0/%s-%s-%i.root' % (name, site, f), size = 1024,
                                     events = 20, checksums = {'cksum': 1})
            testFile.setAlgorithm(appName = name, appVer = "CMSSW_3_1_1",
                                  appFam = "RECO", psetHash = "GIBBERISH",
                                  configContent = "MOREGIBBERISH")
            testFile.setDatasetPath("/%s/%s/%s" % (name, name, tier))
            lumis = []
            for i in range(nLumis):
                lumis.append((f * 100000) + i)
            testFile.addRun(Run( 1, *lumis))
            testFile.setAcquisitionEra(name.split('-')[0])
            testFile.setProcessingVer("0")
            testFile.setGlobalTag("Weird")
            testFile.create()
            testFile.setLocation(site)
            files.append(testFile)

        count = 0
        for f in files:
            count += 1
            testFileChild = DBSBufferFile(lfn = '/data/store/random/random/RANDOM/test/0/%s-%s-%i-child.root' %(name, site, count), size = 1024,
                                          events = 10, checksums = {'cksum': 1})
            testFileChild.setAlgorithm(appName = name, appVer = "CMSSW_3_1_1",
                                       appFam = "RECO", psetHash = "GIBBERISH",
                                       configContent = "MOREGIBBERISH")
            testFileChild.setDatasetPath("/%s/%s_2/RECO" %(name, name))
            testFileChild.addRun(Run( 1, *[45]))
            testFileChild.create()
            testFileChild.setLocation(site)

            testFileChild.addParents([f['lfn']])


        return files



if __name__ == "__main__":
    """
    Run the code

    """

    if False: # Enable test at your own risk
        scaleTester = scaleTestFiller()
        while True:
            print("Ready to begin scale testing")
            scaleTester()
            print("Done running for now, sleeping temporarily")
            time.sleep(random.randint(10, 60))



#!/usr/bin/env python
"""
_GetSpecAndNameFromTask_

Oracle implementation of Workflow.GetSpecAndNameFromTask

Created on Oct 9, 2012

@author: dballest
"""

from WMCore.WMBS.MySQL.Workflow.GetSpecAndNameFromTask import GetSpecAndNameFromTask as MySQLGetSpecAndNameFromTask

class GetSpecAndNameFromTask(MySQLGetSpecAndNameFromTask):
    pass



#!/usr/bin/env python
"""
_JobGroup_t_

Testcase for the JobGroup class.
"""

from WMCore.DataStructs.File import File
from WMCore.DataStructs.Job import Job
from WMCore.DataStructs.JobGroup import JobGroup
from WMCore.DataStructs.Subscription import Subscription

from WMQuality.TestInit import TestInit

import unittest

class JobGroupTest(unittest.TestCase):
    def setUp(self):
        pass

    def testCreate(self):
        """
        _testCreate_

        Test the JobGroup constructor and passing different job containers
        into it.
        """
        testSubscription = Subscription()
        testJobGroupA = JobGroup(subscription = testSubscription)

        assert testJobGroupA.subscription == testSubscription, \
            "ERROR: Failed to pass subscription in constructor"
        assert len(testJobGroupA.jobs) == 0 and len(testJobGroupA.newjobs) == 0, \
            "ERROR: JobGroup not empty on creation"

        testJobA = Job()
        testJobB = Job()

        testJobGroupB = JobGroup(jobs = [testJobA, testJobB])

        assert testJobGroupB.jobs == [], \
            "ERROR: Jobs committed to jobgroup too soon."

        jobGroupJobs = testJobGroupB.newjobs
        goldenJobs = [testJobA, testJobB]
        for job in jobGroupJobs:
            assert job in goldenJobs, \
                "ERROR: Extra job in job group"

            goldenJobs.remove(job)

        assert len(goldenJobs) == 0, \
            "ERROR: Job missing from job group"

        testJobGroupC = JobGroup(jobs = testJobA)

        assert testJobGroupC.jobs == [], \
            "ERROR: Jobs committed to jobgroup too soon."

        jobGroupJobs = testJobGroupC.newjobs

        assert len(jobGroupJobs) == 1, \
            "ERROR: Wrong number of jobs in jobgroup."
        assert testJobA in jobGroupJobs, \
            "ERROR: Wrong job in jobgroup."

        return

    def testAddCommit(self):
        """
        _testAddCommit_

        Test the add() and commit() methods of the JobGroup class.  Verify that
        jobs are not returned from getJobs() until commit() has been called.
        """
        testJob = Job()
        testJobGroup = JobGroup()

        assert len(testJobGroup.getJobs()) == 0, \
            "ERROR: JobGroup has jobs before jobs have been added."

        testJobGroup.add(testJob)

        assert len(testJobGroup.getJobs()) == 0, \
            "ERROR: JobGroup has jobs commit() was called."

        testJobGroup.commit()

        assert len(testJobGroup.getJobs()) == 1, \
            "ERROR: JobGroup has wrong number of jobs."
        assert testJob in testJobGroup.getJobs(), \
            "ERROR: JobGroup has unknown jobs."

        return

    def testAddOutput(self):
        """
        _testAddOutput_

        Test the JobGroup's addOutput() method.  Verify that files are committed
        to the output fileset immediately and are available from the getOutput()
        method.
        """
        testFile = File()
        testJobGroup = JobGroup()

        assert len(testJobGroup.getOutput()) == 0, \
            "ERROR: Files in the output fileset before anything has been added."

        testJobGroup.addOutput(testFile)

        assert len(testJobGroup.getOutput()) == 1, \
            "ERROR: Unknown number of files in JobGroup output fileset."
        assert testFile in testJobGroup.getOutput(), \
            "ERROR: Unknown file in the JobGroup output fileset."

        return

    def testGetJobs(self):
        """
        _testGetJobs_

        Verify that the getJobs() method of the JobGroup class returns the
        correct output for each output container type it supports.
        """
        testJobA = Job()
        testJobB = Job()
        testJobGroup = JobGroup(jobs = [testJobA, testJobB])
        testJobGroup.commit()

        assert len(testJobGroup.getJobs()) == 2,  \
            "ERROR: Wrong number of jobs in job group"

        goldenJobs = [testJobA, testJobB]
        for job in testJobGroup.getJobs():
            assert job in goldenJobs, \
                "ERROR: Unknown Job in JobGroup."

            goldenJobs.remove(job)

        assert len(goldenJobs) == 0, \
            "ERROR: Jobs are missing from the JobGroup."

        goldenIDs = []
        goldenIDs.append(testJobA["id"])
        goldenIDs.append(testJobB["id"])
        for jobID in testJobGroup.getJobs(type = "id"):
            assert jobID in goldenIDs, \
                "ERROR: Unknown JobID in JobGroup"

            goldenIDs.remove(jobID)

        assert len(goldenIDs) == 0, \
            "ERROR: Job IDs are missing from the JobGroup."

        return

    def testLen(self):
        """
        __testLen__

        Test that the __len__ function will actualy return the correct length.

        """

        #This is simple.  It should just have a length equal to the number of committed
        #And yet to be committed jobs

        testJobA = Job()
        testJobB = Job()
        testJobC = Job()
        testJobD = Job()
        testJobGroup = JobGroup(jobs = [testJobA, testJobB])
        testJobGroup.commit()

        self.assertEqual(len(testJobGroup), 2)

        testJobGroup.add(testJobC)

        self.assertEqual(len(testJobGroup), 3)

        testJobGroup.commit()
        testJobGroup.add(testJobD)

        self.assertEqual(len(testJobGroup), 4)

        return


if __name__ == '__main__':
    unittest.main()
